# -*- coding: utf-8 -*-

# ####################################################################
#  Copyright (C) 2005-2019 by the FIFE team
#  http://www.fifengine.net
#  This file is part of FIFE.
#
#  FIFE is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the
#  Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
# ####################################################################

from __future__ import absolute_import
from fife import fifechan

from fife.extensions.pychan.attrs import UnicodeAttr

from .common import gui2text, text2gui
from .widget import Widget


class PasswordField(Widget):
	"""
	An editable B{single line} text edit widget.

	New Attributes
	==============

	  - text: The text in the TextBox.

	Data
	====
	The text can be read and set via L{distributeData} and L{collectData}.
	"""

	ATTRIBUTES = Widget.ATTRIBUTES + [ UnicodeAttr('text') ]
	DEFAULT_HEXPAND = True
	DEFAULT_VEXPAND = False
	DEFAULT_TEXT = u""

	def __init__(self, 
				 parent = None,
				 name = None,
				 size = None,
				 min_size = None,
				 max_size = None,
				 fixed_size = None,
				 margins = None,
				 padding = None,
				 helptext = None,
				 position = None,
				 style = None,
				 hexpand = None,
				 vexpand = None,
				 font = None,
				 base_color = None,
				 background_color = None,
				 foreground_color = None,
				 selection_color = None,
				 border_color = None,
				 outline_color = None,
				 border_size = None,
				 outline_size = None,
				 position_technique = None,
				 is_focusable = None,
				 comment = None,
				 text = None):
				 
		self.real_widget = fifechan.PasswordField()
		self.text = text or self.DEFAULT_TEXT
		super(PasswordField,self).__init__(parent=parent,
										   name=name,
										   size=size,
										   min_size=min_size,
										   max_size=max_size,
										   fixed_size=fixed_size,
										   margins=margins,
										   padding=padding,
										   helptext=helptext,
										   position=position,
										   style=style,
										   hexpand=hexpand,
										   vexpand=vexpand,
										   font=font,
										   base_color=base_color,
										   background_color=background_color,
										   foreground_color=foreground_color,
										   selection_color=selection_color,
										   border_color=border_color,
										   outline_color=outline_color,
										   border_size=border_size,
										   outline_size=outline_size,
										   position_technique=position_technique,
										   is_focusable=is_focusable,
										   comment=comment)

		# Prepare Data collection framework
		self.accepts_data = True
		self.accepts_initial_data = True
		self._realSetInitialData = self._setText
		self._realSetData = self._setText
		self._realGetData = self._getText

	def clone(self, prefix):
		passwordFieldClone = PasswordField(None,  
				self._createNameWithPrefix(prefix),
				self.size,
				self.min_size, 
				self.max_size,
				self.fixed_size,
				self.margins,
				self.padding,
				self.helptext, 
				self.position, 
				self.style, 
				self.hexpand,
				self.vexpand,
				self.font,
				self.base_color,
				self.background_color,
				self.foreground_color,
				self.selection_color,
				self.border_color,
				self.outline_color,
				self.border_size,
				self.outline_size,
				self.position_technique,
				self.is_focusable,
				self.comment,
				self.text)
		return passwordFieldClone
	
	def _getText(self): return gui2text(self.real_widget.getText())
	def _setText(self,text): self.real_widget.setText(text2gui(text))
	text = property(_getText,_setText)

	def _setOpaque(self,opaque): self.real_widget.setOpaque(opaque)
	def _getOpaque(self): return self.real_widget.isOpaque()
	opaque = property(_getOpaque,_setOpaque)



# -*- coding: utf-8 -*-

# ####################################################################
#  Copyright (C) 2005-2013 by the FIFE team
#  http://www.fifengine.net
#  This file is part of FIFE.
#
#  FIFE is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the
#  Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
# ####################################################################

from __future__ import absolute_import
from builtins import range
from fife import fifechan

from fife.extensions.pychan.attrs import BoolAttr, ColorAttr, IntAttr, IntListAttr
from fife.extensions.pychan.properties import ColorProperty

from .widget import Widget


class LineGraph(Widget):
	""" A line graph widget

	New Attributes
	==============

	  - coordinates: int list: x and y coordinates
	  - thickness': int: Line thickness, default 1
	  - opaque: bool: default False
	"""

	ATTRIBUTES = Widget.ATTRIBUTES + [ IntListAttr('coordinates'),
									   IntAttr('thickness'),
									   BoolAttr('opaque')
									 ]
	DEFAULT_HEXPAND = False
	DEFAULT_VEXPAND = False

	DEFAULT_THICKNESS = 1
	DEFAULT_OPAQUE = False

	def __init__(self,
				 parent = None,
				 name = None,
				 size = None,
				 min_size = None,
				 max_size = None,
				 fixed_size = None,
				 margins = None,
				 padding = None,
				 helptext = None,
				 position = None,
				 style = None,
				 hexpand = None,
				 vexpand = None,
				 font = None,
				 base_color = None,
				 background_color = None,
				 foreground_color = None,
				 selection_color = None,
				 border_color = None,
				 outline_color = None,
				 border_size = None,
				 outline_size = None,
				 position_technique = None,
				 is_focusable = None,
				 comment = None,
				 opaque = None,
				 coordinates = None,
				 thickness = None):

		self.real_widget = fifechan.LineGraph()
		self.opaque = self.DEFAULT_OPAQUE
		self.thickness = self.DEFAULT_THICKNESS

		super(LineGraph, self).__init__(parent=parent,
										name=name,
										size=size,
										min_size=min_size,
										max_size=max_size,
										fixed_size=fixed_size,
										margins=margins,
										padding=padding,
										helptext=helptext,
										position=position,
										style=style,
										hexpand=hexpand,
										vexpand=vexpand,
										font=font,
										base_color=base_color,
										background_color=background_color,
										foreground_color=foreground_color,
										selection_color=selection_color,
										border_color=border_color,
										outline_color=outline_color,
										border_size=border_size,
										outline_size=outline_size,
										position_technique=position_technique,
										is_focusable=is_focusable,
										comment=comment)

		if opaque is not None: self.opaque = opaque
		if coordinates is not None:	self.coordinates = coordinates
		if thickness is not None: self.thickness = thickness


	def clone(self, prefix):
		lineGraphClone = LineGraph(None,
					self._createNameWithPrefix(prefix),
					self.size,
					self.min_size,
					self.max_size,
					self.fixed_size,
					self.margins,
					self.padding,
					self.helptext,
					self.position,
					self.style,
					self.hexpand,
					self.vexpand,
					self.font,
					self.base_color,
					self.background_color,
					self.foreground_color,
					self.selection_color,
					self.border_color,
					self.outline_color,
					self.border_size,
					self.outline_size,
					self.position_technique,
					self.is_focusable,
					self.comment,
					self.opaque,
					self.coordinates,
					self.thickness)
		return lineGraphClone

	def _setOpaque(self, opaque): self.real_widget.setOpaque(opaque)
	def _getOpaque(self): return self.real_widget.isOpaque()
	opaque = property(_getOpaque, _setOpaque)

	def _setCoordinates(self, coordinates):
		# reset
		if coordinates == None or len(coordinates) == 0:
			self.real_widget.resetPointVector()
			return
		# int list to point vector
		if isinstance(coordinates[0], int):
			coords = fifechan.FcnPointVector()
			for i in range(0, len(coordinates)-1, 2):
				coords.append(fifechan.Point(coordinates[i], coordinates[i+1]))
			self.real_widget.setPointVector(coords)
			return
		self.real_widget.setPointVector(coordinates)
	def _getCoordinates(self): return self.real_widget.getPointVector()
	coordinates = property(_getCoordinates, _setCoordinates)

	def _setThickness(self, thickness): self.real_widget.setThickness(thickness)
	def _getThickness(self): return self.real_widget.getThickness()
	thickness = property(_getThickness, _setThickness)




# -*- coding: utf-8 -*-
# ####################################################################
#  Copyright (C) 2005-2019 by the FIFE team
#  http://www.fifengine.net
#  This file is part of FIFE.
#
#  FIFE is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the
#  Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
# ####################################################################

""" Loaders plugin manager """
from __future__ import print_function

import os.path

from fife import fife
from fife.extensions.serializers.xmlmap import XMLMapLoader

mapFileMapping = { 'xml' : XMLMapLoader}
fileExtensions = set(['xml'])

def loadMapFile(path, engine, callback=None, debug=True, extensions={}):
	""" load map file and get (an optional) callback if major stuff is done:

		- map creation
		- parsed imports
		- parsed layers
		- parsed cameras

		the callback will send both a string and a float (which shows
		the overall process), callback(string, float)

	@type	engine:		object
	@param	engine: 	FIFE engine instance
	@type	callback:	function
	@param	callback:	callback for maploading progress
	@type	debug:		bool
	@param	debug:		flag to activate / deactivate print statements
	@rtype	object
	@return	FIFE map object
	"""
	(filename, extension) = os.path.splitext(path)
	map_loader = mapFileMapping[extension[1:]](engine, callback, debug, extensions)
	map = map_loader.loadResource(path)
	if debug: print("--- Loading map took: ", map_loader.time_to_load, " seconds.")
	return map

def addMapLoader(fileExtension, loaderClass):
	"""Add a new loader for fileextension
	@type   fileExtension: string
	@param  fileExtension: The file extension the loader is registered for
	@type   loaderClass:   object
	@param  loaderClass:   A fife.ResourceLoader implementation that loads maps
	                       from files with the given fileExtension
	"""
	mapFileMapping[fileExtension] = loaderClass
	_updateMapFileExtensions()

def _updateMapFileExtensions():
	global fileExtensions
	fileExtensions = set(mapFileMapping.keys())



# -*- coding: utf-8 -*-

# ####################################################################
#  Copyright (C) 2005-2019 by the FIFE team
#  http://www.fifengine.net
#  This file is part of FIFE.
#
#  FIFE is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the
#  Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
# ####################################################################

from __future__ import absolute_import
from fife import fifechan

from fife.extensions.pychan.attrs import Attr, BoolAttr, PointAttr, IntAttr
from fife.extensions.pychan.properties import ImageProperty

from .common import text2gui
from .basictextwidget import BasicTextWidget


class Button(BasicTextWidget):
	"""
	A basic push button.
	
	New Attributes
	==============
	
	  - active: Bool: If false the button is inactiv then it looks gray.
	  - alignment: Int: Sets the alignment of the caption. The alignment is relative to the center of the button.
	  - offset: Point: The offset of the caption when the button is pressed.
	"""

	ATTRIBUTES = BasicTextWidget.ATTRIBUTES + [ BoolAttr('active'),
												IntAttr('alignment'),
												PointAttr('offset')
												]

	DEFAULT_ACTIVE = True
	# 0=Left, 1=Center, 2=Right
	DEFAULT_ALIGNMENT = 1
	DEFAULT_OFFSET = 1,1
	
	def __init__(self, 
				 parent = None,
				 name = None,
				 size = None,
				 min_size = None,
				 max_size = None,
				 fixed_size = None,
				 margins = None,
				 padding = None,
				 helptext = None,
				 position = None,
				 style = None,
				 hexpand = None,
				 vexpand = None,
				 font = None,
				 base_color = None,
				 background_color = None,
				 foreground_color = None,
				 selection_color = None,
				 border_color = None,
				 outline_color = None,
				 border_size = None,
				 outline_size = None,
				 position_technique = None,
				 is_focusable = None,
				 comment = None,
				 text = None,
				 active = None,
				 alignment = None,
				 offset = None,
				 real_widget = None):

		if real_widget is None:
			self.real_widget = fifechan.Button("")
		else:
			self.real_widget = real_widget

		# set the defaults
		#offset = self.DEFAULT_OFFSET
		
		super(Button,self).__init__(parent=parent,
									name=name,
									size=size,
									min_size=min_size,
									max_size=max_size,
									fixed_size=fixed_size,
									margins=margins,
									padding=padding,
									helptext=helptext,
									position=position,
									style=style,
									hexpand=hexpand,
									vexpand=vexpand,
									font=font,
									base_color=base_color,
									background_color=background_color,
									foreground_color=foreground_color,
									selection_color=selection_color,
									border_color=border_color,
									outline_color=outline_color,
									border_size=border_size,
									outline_size=outline_size,
									position_technique=position_technique,
									is_focusable=is_focusable,
									comment=comment,
									text=text)

		# set provided attributes or defaults
		if active is not None: self.active = active
		else: self.active = self.DEFAULT_ACTIVE

		if alignment is not None: self.alignment = alignment
		else: self.alignment = self.DEFAULT_ALIGNMENT
		
		if offset is not None: self.offset = offset
		else: self.offset = self.DEFAULT_OFFSET
									
	def clone(self, prefix):
		btnClone = Button(None, 
					self._createNameWithPrefix(prefix),
					self.size,
					self.min_size,
					self.max_size,
					self.fixed_size,
					self.margins,
					self.padding,
					self.helptext,
					self.position,
					self.style,
					self.hexpand,
					self.vexpand,
					self.font,
					self.base_color,
					self.background_color,
					self.foreground_color,
					self.selection_color,
					self.border_color,
					self.outline_color,
					self.border_size,
					self.outline_size,
					self.position_technique,
					self.is_focusable,
					self.comment,
					self.text,
					self.active,
					self.alignment,
					self.offset)
		return btnClone

	def _setActive(self, active): self.real_widget.setActive(active)
	def _isActive(self): return self.real_widget.isActive()
	active = property(_isActive, _setActive)

	def _setAlignment(self, alignment): self.real_widget.setAlignment(alignment)
	def _getAlignment(self): return self.real_widget.getAlignment()
	alignment = property(_getAlignment, _setAlignment)
	
	def _setOffset(self, offset): self.real_widget.setDownOffset(offset[0], offset[1])
	def _getOffset(self): return (self.real_widget.getDownXOffset(), self.real_widget.getDownYOffset())
	offset = property(_getOffset,_setOffset)

	def _setXOffset(self, x): self.real_widget.setDownXOffset(x)
	def _getXOffset(self): return self.real_widget.getDownXOffset()
	x_offset = property(_getXOffset, _setXOffset)

	def _setYOffset(self, y): self.real_widget.setDownYOffset(y)
	def _getYOffset(self): return self.real_widget.getDownYOffset()
	y_offset = property(_getYOffset, _setYOffset)
	

class ImageButton(Button):
	"""
	A basic push button with six different images for the up, down and hover state.

	New Attributes
	==============

	  - up_image: String: The source location of the Image for the B{unpressed} state.
	  - down_image: String: The source location of the Image for the B{pressed} state.
	  - hover_image: String: The source location of the Image for the B{unpressed hovered} state.
	  - in_up_image: String: The source location of the Image for the B{unpressed} inactive state.
	  - in_down_image: String: The source location of the Image for the B{pressed} inactive state.
	  - in_hover_image: String: The source location of the Image for the B{unpressed hovered} inactive state.
	"""

	ATTRIBUTES = Button.ATTRIBUTES + [ Attr('up_image'),
									   Attr('down_image'),
									   Attr('hover_image'),
									   Attr('in_up_image'),
									   Attr('in_down_image'),
									   Attr('in_hover_image')
									]

	#DEFAULT_OFFSET = 0,0
	
	def __init__(self, 
				 parent = None,
				 name = None,
				 size = None,
				 min_size = None,
				 max_size = None,
				 fixed_size = None,
				 margins = None,
				 padding = None,
				 helptext = None,
				 position = None,
				 style = None,
				 hexpand = None,
				 vexpand = None,
				 font = None,
				 base_color = None,
				 background_color = None,
				 foreground_color = None,
				 selection_color = None,
				 border_color = None,
				 outline_color = None,
				 border_size = None,
				 outline_size = None,
				 position_technique = None,
				 is_focusable = None,
				 comment = None,
				 text = None,
				 active = None,
				 alignment = None,
				 offset = None,
				 up_image = None,
				 down_image = None,
				 hover_image = None,
				 in_up_image = None,
				 in_down_image = None,
				 in_hover_image = None,
				 real_widget = None):

		if real_widget is None:
			real_widget = fifechan.ImageButton()
		self.real_widget = real_widget
		
		# set the defaulst
		#offset = self.DEFAULT_OFFSET
		
		super(ImageButton,self).__init__(parent=parent, 
										 name=name,
										 size=size,
										 min_size=min_size,
										 max_size=max_size,
										 fixed_size=fixed_size,
										 margins=margins,
										 padding=padding,
										 helptext=helptext,
										 position=position,
										 style=style,
										 hexpand=hexpand,
										 vexpand=vexpand,
										 font=font,
										 base_color=base_color,
										 background_color=background_color,
										 foreground_color=foreground_color,
										 selection_color=selection_color,
										 border_color=border_color,
										 outline_color=outline_color,
										 border_size=border_size,
										 outline_size=outline_size,
										 position_technique=position_technique,
										 is_focusable=is_focusable,
										 comment=comment,
										 text=text,
										 active=active,
										 alignment=alignment,
										 offset=offset,
										 real_widget=self.real_widget)
		

		# for the case that image can not be found, e.g. invalid path
		# the ImageButton is removed from the manager
		try:
			self.up_image = up_image
			self.down_image = down_image
			self.hover_image = hover_image
			self.in_up_image = in_up_image
			self.in_down_image = in_down_image
			self.in_hover_image = in_hover_image
		except Exception:
			get_manager().removeWidget(self)
			raise
		
		# Override anything set when stylize was called
		#if offset is not None: self.offset = offset

		
	def clone(self, prefix):
		
		imgButtonClone = ImageButton(None, 
						self._createNameWithPrefix(prefix),
						self.size,
						self.min_size,
						self.max_size,
						self.fixed_size,
						self.margins,
						self.padding,
						self.helptext,
						self.position,
						self.style,
						self.hexpand,
						self.vexpand,
						self.font,
						self.base_color,
						self.background_color,
						self.foreground_color,
						self.selection_color,
						self.border_color,
						self.outline_color,
						self.border_size,
						self.outline_size,
						self.position_technique,
						self.is_focusable,
						self.comment,
						self.text,
						self.active,
						self.alignment,
						self.offset,
						self.up_image,
						self.down_image,
						self.hover_image,
						self.in_up_image,
						self.in_down_image,
						self.in_hover_image)
		return imgButtonClone
		
	
	up_image = ImageProperty("UpImage")
	down_image = ImageProperty("DownImage")
	hover_image = ImageProperty("HoverImage")
	in_up_image = ImageProperty("InactiveUpImage")
	in_down_image = ImageProperty("InactiveDownImage")
	in_hover_image = ImageProperty("InactiveHoverImage")


class ToggleButton(ImageButton):
	"""
	A basic push button that can be toggled.

	New Attributes
	==============

	  - group: String: The group the button belongs to. Only one button in each group will be toggled at one time.
	  - toggled: Boolean: Whether the button is toggled or not.

	Data
	====
	The toggled status can be read and set via L{distributeData} and L{collectData}
	"""

	ATTRIBUTES = ImageButton.ATTRIBUTES + [ Attr('group'),
											BoolAttr('toggled')
											]
	DEFAULT_GROUP = ""
	DEFAULT_TOGGLED = False

	def __init__(self, 
				 parent = None,
				 name = None,
				 size = None,
				 min_size = None,
				 max_size = None,
				 fixed_size = None,
				 margins = None,
				 padding = None,
				 helptext = None,
				 position = None,
				 style = None,
				 hexpand = None,
				 vexpand = None,
				 font = None,
				 base_color = None,
				 background_color = None,
				 foreground_color = None,
				 selection_color = None,
				 border_color = None,
				 outline_color = None,
				 border_size = None,
				 outline_size = None,
				 position_technique = None,
				 is_focusable = None,
				 comment = None,
				 text = None,
				 active = None,
				 alignment = None,
				 offset = None,
				 up_image = None,
				 down_image = None,
				 hover_image = None,
				 in_up_image = None,
				 in_down_image = None,
				 in_hover_image = None,
				 group = None,
				 toggled = None):

		self.real_widget = fifechan.ToggleButton()
		
		super(ToggleButton,self).__init__(parent=parent,
										  name=name,
										  size=size,
										  min_size=min_size,
										  max_size=max_size,
										  fixed_size=fixed_size,
										  margins=margins,
										  padding=padding,
										  helptext=helptext,
										  position=position,
										  style=style,
										  hexpand=hexpand,
										  vexpand=vexpand,
										  font=font,
										  base_color=base_color,
										  background_color=background_color,
										  foreground_color=foreground_color,
										  selection_color=selection_color,
										  border_color=border_color,
										  outline_color=outline_color,
										  border_size=border_size,
										  outline_size=outline_size,
										  position_technique=position_technique,
										  is_focusable=is_focusable,
										  comment=comment,
										  text=text,
										  active=active,
										  alignment=alignment,
										  offset=offset,
										  up_image=up_image,
										  down_image=down_image,
										  hover_image=hover_image,
										  in_up_image=in_up_image,
										  in_down_image=in_down_image,
										  in_hover_image=in_hover_image,
										  real_widget=self.real_widget)
										  
		# set provided attributes or defaults
		if group is not None: self.group = group
		else: self.group = self.DEFAULT_GROUP
		
		if toggled is not None: self.toggled = toggled
		else: self.toggled = self.DEFAULT_TOGGLED
		
		# Prepare Data collection framework
		self.accepts_data = True
		self._realGetData = self._isToggled
		self._realSetData = self._setToggled

	def clone(self, prefix):
		toggleButtonClone = ToggleButton(None,
							self._createNameWithPrefix(prefix),
							self.size,
							self.min_size,
							self.max_size,
							self.fixed_size,
							self.margins,
							self.padding,
							self.helptext,
							self.position,
							self.style,
							self.hexpand,
							self.vexpand,
							self.font,
							self.base_color,
							self.background_color,
							self.foreground_color,
							self.selection_color,
							self.border_color,
							self.outline_color,
							self.border_size,
							self.outline_size,
							self.position_technique,
							self.is_focusable,
							self.comment,
							self.text,
							self.active,
							self.alignment,
							self.offset,
							self.up_image,
							self.down_image,
							self.hover_image,
							self.in_up_image,
							self.in_down_image,
							self.in_hover_image,
							self.group,
							self.toggled)
		return toggleButtonClone
		
	def _setGroup(self, group):
		if group is None: group = ""
		self.real_widget.setGroup(group)
	def _getGroup(self): return self.real_widget.getGroup()
	group = property(_getGroup,_setGroup)

	def _setToggled(self, toggled): self.real_widget.setSelected(toggled)
	def _isToggled(self): return self.real_widget.isSelected()
	toggled = property(_isToggled, _setToggled)



# -*- coding: utf-8 -*-

# ####################################################################
#  Copyright (C) 2005-2019 by the FIFE team
#  http://www.fifengine.net
#  This file is part of FIFE.
#
#  FIFE is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the
#  Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
# ####################################################################

from future import standard_library
standard_library.install_aliases()
from builtins import str
from past.builtins import basestring
from builtins import object
import os
from io import BytesIO, StringIO
	
from fife.extensions.serializers import ET, SerializerError, InvalidFormat, \
										NotFound


EMPTY_XML_FILE="""\
<?xml version='1.0' encoding='UTF-8'?>
<Settings>

</Settings>
"""

class SimpleSerializer(object):
	"""
	Use this as a base class for custom setting loaders/savers to use with the 
	Setting class.
	"""
	
	def __init__(self, filename=None):
		pass
		
	def get(self, module, name, defaultValue=None):
		pass
		
	def set(self, module, name, value, extra_attrs={}):
		pass

	def set(self, module, name, value, extra_attrs={}):
		pass

	def load(self, filename=None):
		"""
		@note: If the filename specified is empty this function MUST 
		initialize an empty settings file in whatever format you need.
		"""
		pass
		
	def save(self, filename=None):
		pass

	def getModuleNameList(self):
		"""
		@note: Returns all the module names that are present in the 
		settings.xml file as a list of strings
		"""
		pass
	
	def getAllSettings(self,module):
		"""
		@note: Returns all the setting names and values under the Module name
		module as a	dictionary structure
		"""
		pass

class SimpleXMLSerializer(SimpleSerializer):
	"""
	This class is a simple interface to get and store data in XML files.

	Usage::
		from fife.extensions.serializers.simplexml import SimpleXMLSerializer
		serializer = SimpleXMLSerializer(filename="somefile.xml")
		serializer.set("module_name", "variable_name", "value")
		somevariable = serializer.get("module_name", "variable_name", \
									  "default_value")
	"""
	def __init__(self, filename=None):
		self._file = filename
		self._tree = None
		self._root_element = None
		
		self._initialized = False
		
	def load(self, filename=None):
		"""
		Loads the XML file into memory and validates it.
		
		Raises a SerializerError exception if the file is not specified.
		
		@param filename: The file to load
		@type filename: C{str}
		
		@note: If the file does not exist it will automatically create a blank
		file for you.
		"""
		if filename:
			self._file = filename
		
		if not self._file:
			raise SerializerError("Cannot load file or create file.  No "
								  "filename specified!")
		
		if not os.path.exists(self._file):
			try:
				self._tree = ET.parse(BytesIO(EMPTY_XML_FILE))
			except TypeError:
				self._tree = ET.parse(StringIO(EMPTY_XML_FILE))
			self._tree.write(self._file, 'UTF-8')			
		else:
			self._tree = ET.parse(self._file)

		self._root_element = self._tree.getroot()
		self._validateTree()

	def save(self, filename=None):
		"""
		Saves the XML file.
		
		@param filename: The file to save
		@type filename: C{str}
		
		@note: This Overwrites the file if it exists.
		"""
		if not self._initialized:
			self.load()	
			self._initialized = True
		
		if filename:
			savefile = filename
		else:
			savefile = self._file
				
		if not savefile:
			raise SerializerError("Cannot save file.  No filename specified!")
	
		# Writes the settings to file
		self._indent(self._root_element)
		self._tree.write(savefile, 'UTF-8')


	def getValue(self, e_type, e_value):
		if e_type == 'int':
			return int(e_value)
		elif e_type == 'float':
			return float(e_value)
		elif e_type == 'bool':
			e_value = e_value.lower()
			if e_value == "" or e_value == "false" or e_value == "no" \
			   or e_value == "0":
				return False
			else:
				return True
		elif e_type == 'str' or e_type == 'string':
			return str(e_value)
		elif e_type == 'unicode':
			return str(e_value)
		elif e_type == 'list':
			return self._deserializeList(e_value)
		elif e_type == 'dict':
			return self._deserializeDict(e_value)
		

	def get(self, module, name, defaultValue=None):
		""" Gets the value of a specified variable

		@param module: Name of the module to get the variable from
		@param name: Variable name
		@param defaultValue: Specifies the default value to return if the 
		variable is not found
		@type defaultValue: C{str} or C{unicode} or C{int} or C{float} or 
		C{bool} or C{list} or C{dict}
		"""
		if not self._initialized:
			self.load()
			self._initialized = True
		
		if not isinstance(name, basestring):
			raise AttributeError("SimpleXMLSerializer.get(): Invalid type for "
								 "name argument.")

		#get the module tree: for example find tree under module FIFE
		moduleTree = self._getModuleTree(module)
		element = None
		for e in moduleTree:
			if e.tag == "Setting" and e.get("name", "") == name:
				element = e
				break
		else:
			return defaultValue

		e_value = element.text
		e_strip = element.get("strip", "1").strip().lower()
		e_type	= str(element.get("type", "str")).strip()

		if e_value is None:
			return defaultValue

		# Strip value
		if e_strip == "" or e_strip == "false" or e_strip == "no" \
		   or e_strip == "0":
			e_strip = False
		else: e_strip = True

		if e_type == "str" or e_type == "unicode":
			if e_strip: e_value = e_value.strip()
		else:
			e_value = e_value.strip()

		# Return value
		e_value = self.getValue(e_type,e_value)
		
		return e_value

	def set(self, module, name, value, extra_attrs={}):
		"""
		Sets a variable to specified value.

		@param module: Module where the variable should be set
		@param name: Name of the variable
		@param value: Value to assign to the variable
		@type value: C{str} or C{unicode} or C{int} or C{float} or C{bool} or 
		C{list} or C{dict}
		@param extra_attrs: Extra attributes to be stored in the XML-file
		@type extra_attrs: C{dict}
		"""
		if not self._initialized:
			self.load()
			self._initialized = True
		
		if not isinstance(name, basestring):
			raise AttributeError("SimpleXMLSerializer.set(): Invalid type for "
								 "name argument.")

		moduleTree = self._getModuleTree(module)
		e_type = "str"

		if isinstance(value, bool): # This must be before int
			e_type = "bool"
			value = str(value)
		elif isinstance(value, int):
			e_type = "int"
			value = str(value)
		elif isinstance(value, float):
			e_type = "float"
			value = str(value)
		elif isinstance(value, str):
			e_type = "unicode"
			value = str(value)
		elif isinstance(value, list):
			e_type = "list"
			value = self._serializeList(value)
		elif isinstance(value, dict):
			e_type = "dict"
			value = self._serializeDict(value)
		else:
			e_type = "str"
			value = str(value)

		for e in moduleTree:
			if e.tag != "Setting": continue
			if e.get("name", "") == name:
				e.text = value
				break
		else:
			attrs = {"name":name, "type":e_type}
			for k in extra_attrs:
				if k not in attrs:
					attrs[k] = extra_attrs[k]
			elm = ET.SubElement(moduleTree, "Setting", attrs)
			elm.text = value

	def remove(self, module, name):
		"""
		Removes a variable

		@param module: Module where the variable should be set
		@param name: Name of the variable
		"""
		if not self._initialized:
			self.load()
			self._initialized = True

		if not isinstance(name, basestring):
			raise AttributeError("SimpleXMLSerializer.set(): Invalid type for "
								 "name argument.")

		moduleTree = self._getModuleTree(module)

		for e in moduleTree:
			if e.tag != "Setting": continue
			if e.get("name", "") == name:
				moduleTree.remove(e)

	def getModuleNameList(self):
		"""
		@return A list of the names of the modules in the XML file as strings.
		"""
		
		# Make sure the file has been loaded, if not load it.
		if not self._initialized:
			self.load()
			self._initialized = True

		moduleNames = []
		for c in self._root_element:
			if c.tag == "Module":
				name = c.get("name","")
				if not isinstance(name, basestring):
					raise AttributeError("SimpleXMLSerializer.get(): Invalid "
										 "type for name argument.")
				
				moduleNames.append(name)
		return moduleNames
		
	def getAllSettings(self, module):
		settingsFromFile = {}
		
		# if file has not been loaded, load the file
		if not self._initialized:
			self.load()
			self._initialized = True
			
		# get the module tree, as we want to get values for module FIFE only
		moduleTree = self._getModuleTree(module)
		
		# now from the tree read every value, and put the necessary values
		# to the list
		for e in moduleTree:
			if e.tag == "Setting":
				name = e.get("name", "")
	
				# check the name
				if not isinstance(name, basestring):
					raise AttributeError("SimpleXMLSerializer.get(): Invalid "
										 "type for name argument.")
				element = e
				
				e_value = element.text
				e_strip = element.get("strip", "1").strip().lower()
				e_type	= str(element.get("type", "str")).strip()
					
				# Strip value
				if e_strip == "" or e_strip == "false" \
				   or e_strip == "no" or e_strip == "0":
					e_strip = False
				else: e_strip = True
			
				if e_type == "str" or e_type == "unicode":
					if e_strip and e_value:
						e_value = e_value.strip()
				else:
					if e_value:
						e_value = e_value.strip()
					
				# get the value
				e_value = self.getValue(e_type,e_value)
				settingsFromFile[name] = e_value
		
		return settingsFromFile				

	def _validateTree(self):
		""" 
		Iterates the XML tree and prints warning when an invalid tag is found.
		
		Raises an InvalidFormat exception if there is a format error.
		"""
		for c in self._root_element:
			if c.tag != "Module":
				raise InvalidFormat("Invalid tag in " + self._file + \
									". Expected Module, got: " + c.tag)
			elif c.get("name", "") == "":
				raise InvalidFormat("Invalid tag in " + self._file + \
									". Module name is empty.")
			else:
				for e in c:
					if e.tag != "Setting":
						raise InvalidFormat("Invalid tag in " + self._file + \
											" in module: " + c.tag + \
											". Expected Setting, got: " + \
											e.tag)
					elif c.get("name", "") == "":
						raise InvalidFormat("Invalid tag in " + self._file + \
											" in module: " + c.tag + \
											". Setting name is empty" + e.tag)
						
	def _getModuleTree(self, module):
		"""
		Returns a module element from the XML tree. If no module with the 
		specified name exists, a new element will be created.

		@param module: The module to get from the settings tree
		@type module: C{string}
		"""
		if not isinstance(module, basestring):
			raise AttributeError("Settings:_getModuleTree: Invalid type for "
								 "module argument.")

		for c in self._root_element:
			if c.tag == "Module" and c.get("name", "") == module:
				return c

		# Create module
		return ET.SubElement(self._root_element, "Module", {"name":module})

	def _indent(self, elem, level=0):
		"""
		Adds whitespace, so the resulting XML-file is properly indented.
		Shamelessly stolen from http://effbot.org/zone/element-lib.htm
		"""
		i = os.linesep + level*"  "
		if len(elem):
			if not elem.text or not elem.text.strip():
				elem.text = i + "  "
			if not elem.tail or not elem.tail.strip():
				elem.tail = i
			for elem in elem:
				self._indent(elem, level+1)
			if not elem.tail or not elem.tail.strip():
				elem.tail = i
		else:
			if level and (not elem.tail or not elem.tail.strip()):
				elem.tail = i

	# FIXME:
	# These serialization functions are not reliable at all
	# This will only serialize the first level of a dict or list
	# It will not check the types nor the content for conflicts.
	# Perhaps we should add a small serialization library?
	def _serializeList(self, list):
		""" Serializes a list, so it can be stored in a text file """
		return " ; ".join(list)

	def _deserializeList(self, string):
		""" Deserializes a list back into a list object """
		if not string:
			return  list()
		return string.split(" ; ")

	def _serializeDict(self, dict):
		""" Serializes a list, so it can be stored in a text file """
		serial = ""
		for key in dict:
			value = dict[key]
			if serial != "": serial += " ; "
			serial += str(key)+" : "+str(value)

		return serial

	def _deserializeDict(self, serial):
		""" Deserializes a list back into a dict object """
		if not serial:
			return dict()
		dict = {}
		items = serial.split(" ; ")
		for i in items:
			kv_pair = i.split(" : ")
			dict[kv_pair[0]] = kv_pair[1]
		return dict
		



# -*- coding: utf-8 -*-

# ####################################################################
#  Copyright (C) 2005-2019 by the FIFE team
#  http://www.fifengine.net
#  This file is part of FIFE.
#
#  FIFE is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the
#  Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
# ####################################################################

from builtins import zip
from builtins import str
from builtins import range
from builtins import object
import os
from xml.sax.saxutils import XMLGenerator
from xml.sax.xmlreader import AttributesNSImpl
from fife.extensions.serializers import *

from fife import fife

MAPFORMAT = '1.0'

fileExtensions = ('xml',)
class XMLMapSaver(object):

	def __init__(self, filepath, engine, map, importList, state = 0, datastate = 0):
		self.SModel, self.SMap, self.SLayer, self.SInstances, self.SObject, self.SAction = list(range(6))

		self.engine = engine
		self.model = self.engine.getModel()
		self.image_manager = self.engine.getImageManager()

		if (state):
			self.state = state
			if (state == self.SMap):
				self.map = datastate
			else:
				assert 0, "Invalid initialization state."
		else:
			self.state = self.SModel

		self.stack = [ self.SModel ]
		self.datastack = [ ]

		self.file = open(filepath, 'w')
		self.xmlout = XMLGenerator(self.file, 'ascii')
		self.xmlout.startDocument()

		self.indent_level = ''

		self.nspace = None

		self.map = map
		self.importList = importList

	def startElement(self, name, attrs):
		self.file.write(self.indent_level)
		self.xmlout.startElementNS((None, name), name, attrs)
		self.file.write('\n')
		self.indent_level = self.indent_level + '\t'

	def endElement(self, name):
		self.indent_level = self.indent_level[0:(len(self.indent_level) - 1)]
		self.file.write(self.indent_level)
		self.xmlout.endElementNS((None, name), name)
		self.file.write('\n')

	def write_map(self):
		assert self.state == self.SModel, "Declaration of <map> not at the top level."

		attr_vals = {
			(None, 'id'): self.map.getId(),
			(None, 'format'): MAPFORMAT,
		}
		attr_names = {
			(None, 'id'): 'id',
			(None, 'format'): 'format',
		}
		attrs = AttributesNSImpl(attr_vals, attr_names)
		self.startElement('map', attrs)
		self.state = self.SMap
		self.write_imports(self.map, self.importList)
		self.write_layers(self.map)
		self.write_camera(self.map)
		self.endElement('map')

	def write_imports(self, map, importList):
		for importdir in importList:
			self.write_importdir(root_subfile(map.getFilename(), importdir))

		imports = []
		for layer in map.getLayers():
			for instance in layer.getInstances():
				file = instance.getObject().getFilename()
				if not (file in imports):
					if not self.have_superdir(file, importList):
						imports.append(file)
						self.write_import(root_subfile(map.getFilename(), file))

	def have_superdir(self, file, importList):
		'''returns true, if file is in directories given in importList'''
		for dir in importList:
			have = True
			for test in zip(dir.split(os.path.sep), file.split(os.path.sep)):
				if test[0] != test[1]: have = False
			if have: return True

		return False

	def write_import(self, file):
		attr_vals = {
			(None, 'file'): file,
		}
		attr_names = {
			(None, 'file'): 'file',
		}
		attrs = AttributesNSImpl(attr_vals, attr_names)
		self.file.write(self.indent_level)
		self.xmlout.startElementNS((None, 'import'), 'import', attrs)
		self.xmlout.endElementNS((None, 'import'), 'import')
		self.file.write('\n')

	def write_importdir(self, dir):
		attr_vals = {
			(None, 'dir'): dir,
		}
		attr_names = {
			(None, 'dir'): 'dir',
		}
		attrs = AttributesNSImpl(attr_vals, attr_names)
		self.file.write(self.indent_level)
		self.xmlout.startElementNS((None, 'import'), 'import', attrs)
		self.xmlout.endElementNS((None, 'import'), 'import')
		self.file.write('\n')

	def pathing_val_to_str(self, val):
		if val == fife.CELL_EDGES_AND_DIAGONALS:
			return "cell_edges_and_diagonals"
		if val == fife.FREEFORM:
			return "freeform"
		return "cell_edges_only"

	def layer_type_to_str(self, layer):
		if layer.isWalkable(): return "walkable"
		elif layer.isInteract(): return "interact"
		return ""

	def write_layers(self, map):
		for layer in map.getLayers():
			cellgrid = layer.getCellGrid()
			attr_vals = {
				(None, 'id'): layer.getId(),
				(None, 'grid_type'): cellgrid.getType(),
				(None, 'x_scale'): str(cellgrid.getXScale()),
				(None, 'y_scale'): str(cellgrid.getYScale()),
				(None, 'rotation'): str(cellgrid.getRotation()),
				(None, 'x_offset'): str(cellgrid.getXShift()),
				(None, 'y_offset'): str(cellgrid.getYShift()),
				(None, 'z_offset'): str(cellgrid.getZShift()),
				(None, 'pathing'): self.pathing_val_to_str(layer.getPathingStrategy()),
				(None, 'transparency'): str(layer.getLayerTransparency()),
				(None, 'layer_type'): str(self.layer_type_to_str(layer)),
				(None, 'layer_type_id'): str(layer.getWalkableId()),
			}
			attr_names = {
				(None, 'id'): 'id',
				(None, 'grid_type'): 'grid_type',
				(None, 'scaling'): 'scaling',
				(None, 'rotation'): 'rotation',
				(None, 'x_offset'): 'x_offset',
				(None, 'y_offset'): 'y_offset',
				(None, 'z_offset'): 'z_offset',
				(None, 'pathing'): 'pathing',
				(None, 'layer_type'): 'layer_type',
				(None, 'layer_type_id'): 'layer_type_id',
			}
			attrs = AttributesNSImpl(attr_vals, attr_names)
			self.startElement('layer', attrs)
			self.write_instances(layer)
			self.write_lights(layer)
			self.endElement('layer')

	def write_instances(self, layer):
		attrs = AttributesNSImpl({}, {})
		self.startElement('instances',  attrs)

		for inst in layer.getInstances():
			position = inst.getLocationRef().getExactLayerCoordinates()

			attr_vals = {
				(None, 'o'): inst.getObject().getId(),
				(None, 'x'): str(position.x),
				(None, 'y'): str(position.y),
				(None, 'z'): str(position.z),
				(None, 'r'): str(inst.getRotation()),
			}
			attr_names = {
				(None, 'o'): 'o',
				(None, 'x'): 'x',
				(None, 'y'): 'y',
				(None, 'z'): 'z',
				(None, 'r'): 'r',
			}

			visual = inst.get2dGfxVisual();
			if visual:
				attr_vals[(None, 'stackpos')] = str(visual.getStackPosition())
				attr_names[(None, 'stackpos')] = 'stackpos'

			nspace = inst.getObject().getNamespace()
			if nspace != self.nspace:
				attr_vals[(None, 'ns')] = inst.getObject().getNamespace()
				attr_names[(None, 'ns')] = 'ns'
				self.nspace = nspace

			instId = inst.getId()
			if instId:
				attr_vals[(None, 'id')] = inst.getId()
				attr_names[(None, 'id')] = 'id'

			if inst.isOverrideBlocking():
				attr_vals[(None, 'override_blocking')] = str(int(inst.isOverrideBlocking()))
				attr_names[(None, 'override_blocking')] = 'override_blocking'
				if inst.getObject().isBlocking() is not inst.isBlocking():
					attr_vals[(None, 'blocking')] = str(int(inst.isBlocking()))
					attr_names[(None, 'blocking')] = 'blocking'

			attrs = AttributesNSImpl(attr_vals, attr_names)
			self.file.write(self.indent_level)
			self.xmlout.startElementNS((None, 'i'), 'i', attrs)
			self.xmlout.endElementNS((None, 'i'), 'i')
			self.file.write('\n')

		self.endElement('instances')

	def write_lights(self, layer):
		attrs = AttributesNSImpl({}, {})
		self.startElement('lights',  attrs)

		cameras = layer.getMap().getCameras()
		for cam in cameras:
			hit = False
			layers = cam.getRenderer("LightRenderer").getActiveLayers();
			for lay in layers:
				if lay.getId() == layer.getId():
					hit = True

			if hit == False: continue

			renderer = fife.LightRenderer.getInstance(cam)
			groups = renderer.getGroups()
			for group in groups:
				infos = renderer.getLightInfo(group)
				for info in infos:
					attr_vals = {}
					attr_names = {}
					type = info.getName()
					attr_vals[(None, 'group')] = str(group)
					attr_names[(None, 'group')] = 'group'
					attr_vals[(None, 'type')] = str(type)
					attr_names[(None, 'type')] = 'type'
					attr_vals[(None, 'instance')] = str(info.getNode().getInstance().getId())
					attr_names[(None, 'instance')] = 'instance'
					if info.getSrcBlend() > -1:
						attr_vals[(None, 'src')] = str(info.getSrcBlend())
						attr_names[(None, 'src')] = 'src'
					if info.getDstBlend() > -1:
						attr_vals[(None, 'dst')] = str(info.getDstBlend())
						attr_names[(None, 'dst')] = 'dst'
					if info.getStencil() > -1:
						attr_vals[(None, 's_ref')] = str(info.getStencil())
						attr_names[(None, 's_ref')] = 's_ref'
						attr_vals[(None, 'a_ref')] = str(info.getAlpha())
						attr_names[(None, 'a_ref')] = 'a_ref'

					if type == 'simple':
						if info.getRadius() > 0:
							attr_vals[(None, 'radius')] = str(info.getRadius())
							attr_names[(None, 'radius')] = 'radius'
						if info.getColor():
							color = info.getColor()
							attr_vals[(None, 'color')] = '%d,%d,%d' % (color[0], color[1], color[2])
							attr_vals[(None, 'intensity')] = str(color[3])
							attr_names[(None, 'color')] = 'color'
							attr_names[(None, 'intensity')] = 'intensity'

						if info.getSubdivisions() != 32:
							attr_vals[(None, 'subdivisions')] = str(info.getSubdivisions())
							attr_names[(None, 'subdivisions')] = 'subdivisions'
						if info.getXStretch() > 1.001 or info.getXStretch() < 0.999:
							attr_vals[(None, 'xstretch')] = str(info.getXStretch())
							attr_names[(None, 'xstretch')] = 'xstretch'
						if info.getYStretch() > 1.001 or info.getYStretch() < 0.999:
							attr_vals[(None, 'ystretch')] = str(info.getYStretch())
							attr_names[(None, 'ystretch')] = 'ystretch'

					elif type == 'image':
						if info.getImage() == 0: continue
						img = info.getImage()
						name = img.getName()
						attr_vals[(None, 'image')] = str('../' + name)
						attr_names[(None, 'image')] = 'image'

					elif type == 'animation':
						if info.getAnimation() == 0: continue
						ani = info.getAnimation();
						count = 0
						newstr = ''
						image = ani.getFrame(ani.getActionFrame())
						fname = image.getName()
						strings = ([str(s) for s in fname.split('/')])
						leng = len(strings) -1
						while count < leng:
							newstr = str(newstr + strings[count] + '/')
							count += 1

						name = str('../' + newstr + 'animation.' + fileExtensions[0])
						attr_vals[(None, 'animation')] = str(name)
						attr_names[(None, 'animation')] = 'animation'

					else:
						continue

					attrs = AttributesNSImpl(attr_vals, attr_names)
					self.file.write(self.indent_level)
					self.xmlout.startElementNS((None, 'l'), 'l', attrs)
					self.xmlout.endElementNS((None, 'l'), 'l')
					self.file.write('\n')

		self.endElement('lights')

	# Save the linked camera of a map.
	def write_camera( self, map ):
		cameralist = map.getCameras()

		for cam in cameralist:
			if cam.getMap().getId() == map.getId():
				celldimensions = cam.getCellImageDimensions()
				viewport = cam.getViewPort();

				attr_names = {
						(None, 'id'): 'id',
						(None, 'zoom'): 'zoom',
						(None, 'tilt'): 'tile',
						(None, 'rotation'): 'rotation',
						(None, 'ref_cell_width'): 'ref_cell_width',
						(None, 'ref_cell_height'): 'ref_cell_height',
				}

				attr_vals = {
					(None, 'id'): cam.getId(),
					(None, 'zoom'): str( cam.getZoom()),
					(None, 'tilt'): str( cam.getTilt()),
					(None, 'rotation'): str( cam.getRotation()),
					(None, 'ref_cell_width'): str( celldimensions.x ),
					(None, 'ref_cell_height'): str( celldimensions.y ),
				}

				# add a viewport entry if the cam isn't full sized
				if not (viewport == self.engine.getRenderBackend().getArea()):
					attr_names[(None,'viewport')] = 'viewport'
					attr_vals[(None,'viewport')] = '%d,%d,%d,%d' % (viewport.x, viewport.y, viewport.w, viewport.h)

				colors = cam.getLightingColor()
				if colors[0] < 1.0 or colors[1] < 1.0 or colors[2] < 1.0:
					attr_names[(None,'light_color')] = 'light_color'
					attr_vals[(None,'light_color')] = '%f,%f,%f' % (colors[0], colors[1], colors[2])
				attrs = AttributesNSImpl( attr_vals, attr_names )
				self.startElement( 'camera', attrs );
				self.endElement( 'camera' );

	def flush(self):
		self.xmlout.endDocument()
		self.file.close()

	def saveResource(self):
		self.write_map()



# This file was automatically generated by SWIG (https://www.swig.org).
# Version 4.2.1
#
# Do not make changes to this file unless you know what you are doing - modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _fifechan
else:
    import _fifechan

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _fifechan.SWIG_PyInstanceMethod_New
_swig_new_static_method = _fifechan.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "this":
            set(self, name, value)
        elif name == "thisown":
            self.this.own(value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _fifechan.delete_SwigPyIterator
    value = _swig_new_instance_method(_fifechan.SwigPyIterator_value)
    incr = _swig_new_instance_method(_fifechan.SwigPyIterator_incr)
    decr = _swig_new_instance_method(_fifechan.SwigPyIterator_decr)
    distance = _swig_new_instance_method(_fifechan.SwigPyIterator_distance)
    equal = _swig_new_instance_method(_fifechan.SwigPyIterator_equal)
    copy = _swig_new_instance_method(_fifechan.SwigPyIterator_copy)
    next = _swig_new_instance_method(_fifechan.SwigPyIterator_next)
    __next__ = _swig_new_instance_method(_fifechan.SwigPyIterator___next__)
    previous = _swig_new_instance_method(_fifechan.SwigPyIterator_previous)
    advance = _swig_new_instance_method(_fifechan.SwigPyIterator_advance)
    __eq__ = _swig_new_instance_method(_fifechan.SwigPyIterator___eq__)
    __ne__ = _swig_new_instance_method(_fifechan.SwigPyIterator___ne__)
    __iadd__ = _swig_new_instance_method(_fifechan.SwigPyIterator___iadd__)
    __isub__ = _swig_new_instance_method(_fifechan.SwigPyIterator___isub__)
    __add__ = _swig_new_instance_method(_fifechan.SwigPyIterator___add__)
    __sub__ = _swig_new_instance_method(_fifechan.SwigPyIterator___sub__)
    def __iter__(self):
        return self

# Register SwigPyIterator in _fifechan:
_fifechan.SwigPyIterator_swigregister(SwigPyIterator)
class MouseListener(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    __swig_destroy__ = _fifechan.delete_MouseListener
    mouseEntered = _swig_new_instance_method(_fifechan.MouseListener_mouseEntered)
    mouseExited = _swig_new_instance_method(_fifechan.MouseListener_mouseExited)
    mousePressed = _swig_new_instance_method(_fifechan.MouseListener_mousePressed)
    mouseReleased = _swig_new_instance_method(_fifechan.MouseListener_mouseReleased)
    mouseClicked = _swig_new_instance_method(_fifechan.MouseListener_mouseClicked)
    mouseWheelMovedUp = _swig_new_instance_method(_fifechan.MouseListener_mouseWheelMovedUp)
    mouseWheelMovedDown = _swig_new_instance_method(_fifechan.MouseListener_mouseWheelMovedDown)
    mouseWheelMovedRight = _swig_new_instance_method(_fifechan.MouseListener_mouseWheelMovedRight)
    mouseWheelMovedLeft = _swig_new_instance_method(_fifechan.MouseListener_mouseWheelMovedLeft)
    mouseMoved = _swig_new_instance_method(_fifechan.MouseListener_mouseMoved)
    mouseDragged = _swig_new_instance_method(_fifechan.MouseListener_mouseDragged)

    def __init__(self):
        if self.__class__ == MouseListener:
            _self = None
        else:
            _self = self
        _fifechan.MouseListener_swiginit(self, _fifechan.new_MouseListener(_self, ))
    def __disown__(self):
        self.this.disown()
        _fifechan.disown_MouseListener(self)
        return weakref.proxy(self)

# Register MouseListener in _fifechan:
_fifechan.MouseListener_swigregister(MouseListener)
class KeyListener(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    __swig_destroy__ = _fifechan.delete_KeyListener
    keyPressed = _swig_new_instance_method(_fifechan.KeyListener_keyPressed)
    keyReleased = _swig_new_instance_method(_fifechan.KeyListener_keyReleased)

    def __init__(self):
        if self.__class__ == KeyListener:
            _self = None
        else:
            _self = self
        _fifechan.KeyListener_swiginit(self, _fifechan.new_KeyListener(_self, ))
    def __disown__(self):
        self.this.disown()
        _fifechan.disown_KeyListener(self)
        return weakref.proxy(self)

# Register KeyListener in _fifechan:
_fifechan.KeyListener_swigregister(KeyListener)
class ActionListener(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    __swig_destroy__ = _fifechan.delete_ActionListener
    action = _swig_new_instance_method(_fifechan.ActionListener_action)

    def __init__(self):
        if self.__class__ == ActionListener:
            _self = None
        else:
            _self = self
        _fifechan.ActionListener_swiginit(self, _fifechan.new_ActionListener(_self, ))
    def __disown__(self):
        self.this.disown()
        _fifechan.disown_ActionListener(self)
        return weakref.proxy(self)

# Register ActionListener in _fifechan:
_fifechan.ActionListener_swigregister(ActionListener)
class WidgetListener(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    __swig_destroy__ = _fifechan.delete_WidgetListener
    widgetResized = _swig_new_instance_method(_fifechan.WidgetListener_widgetResized)
    widgetMoved = _swig_new_instance_method(_fifechan.WidgetListener_widgetMoved)
    widgetHidden = _swig_new_instance_method(_fifechan.WidgetListener_widgetHidden)
    widgetShown = _swig_new_instance_method(_fifechan.WidgetListener_widgetShown)
    ancestorMoved = _swig_new_instance_method(_fifechan.WidgetListener_ancestorMoved)
    ancestorHidden = _swig_new_instance_method(_fifechan.WidgetListener_ancestorHidden)
    ancestorShown = _swig_new_instance_method(_fifechan.WidgetListener_ancestorShown)

    def __init__(self):
        if self.__class__ == WidgetListener:
            _self = None
        else:
            _self = self
        _fifechan.WidgetListener_swiginit(self, _fifechan.new_WidgetListener(_self, ))
    def __disown__(self):
        self.this.disown()
        _fifechan.disown_WidgetListener(self)
        return weakref.proxy(self)

# Register WidgetListener in _fifechan:
_fifechan.WidgetListener_swigregister(WidgetListener)
getVersion = _fifechan.getVersion
getMajor = _fifechan.getMajor
getMinor = _fifechan.getMinor
getPatch = _fifechan.getPatch
getHash = _fifechan.getHash
class Graphics(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr
    Left = _fifechan.Graphics_Left
    Center = _fifechan.Graphics_Center
    Right = _fifechan.Graphics_Right
    __swig_destroy__ = _fifechan.delete_Graphics

# Register Graphics in _fifechan:
_fifechan.Graphics_swigregister(Graphics)
class Color(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.Color_swiginit(self, _fifechan.new_Color(*args))
    __add__ = _swig_new_instance_method(_fifechan.Color___add__)
    __sub__ = _swig_new_instance_method(_fifechan.Color___sub__)
    __mul__ = _swig_new_instance_method(_fifechan.Color___mul__)
    __eq__ = _swig_new_instance_method(_fifechan.Color___eq__)
    __ne__ = _swig_new_instance_method(_fifechan.Color___ne__)
    r = property(_fifechan.Color_r_get, _fifechan.Color_r_set)
    g = property(_fifechan.Color_g_get, _fifechan.Color_g_set)
    b = property(_fifechan.Color_b_get, _fifechan.Color_b_set)
    a = property(_fifechan.Color_a_get, _fifechan.Color_a_set)
    __swig_destroy__ = _fifechan.delete_Color

# Register Color in _fifechan:
_fifechan.Color_swigregister(Color)
class Size(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, width=0, height=0):
        _fifechan.Size_swiginit(self, _fifechan.new_Size(width, height))
    __swig_destroy__ = _fifechan.delete_Size
    getWidth = _swig_new_instance_method(_fifechan.Size_getWidth)
    getHeight = _swig_new_instance_method(_fifechan.Size_getHeight)
    setWidth = _swig_new_instance_method(_fifechan.Size_setWidth)
    setHeight = _swig_new_instance_method(_fifechan.Size_setHeight)

# Register Size in _fifechan:
_fifechan.Size_swigregister(Size)
class Point(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    x = property(_fifechan.Point_x_get, _fifechan.Point_x_set)
    y = property(_fifechan.Point_y_get, _fifechan.Point_y_set)

    def __init__(self, *args):
        _fifechan.Point_swiginit(self, _fifechan.new_Point(*args))
    __add__ = _swig_new_instance_method(_fifechan.Point___add__)
    __sub__ = _swig_new_instance_method(_fifechan.Point___sub__)
    __iadd__ = _swig_new_instance_method(_fifechan.Point___iadd__)
    __isub__ = _swig_new_instance_method(_fifechan.Point___isub__)
    __mul__ = _swig_new_instance_method(_fifechan.Point___mul__)

    def __truediv__(self, *args):
        return _fifechan.Point___truediv__(self, *args)
    __div__ = __truediv__


    __eq__ = _swig_new_instance_method(_fifechan.Point___eq__)
    __ne__ = _swig_new_instance_method(_fifechan.Point___ne__)
    length = _swig_new_instance_method(_fifechan.Point_length)
    normalize = _swig_new_instance_method(_fifechan.Point_normalize)
    rotate = _swig_new_instance_method(_fifechan.Point_rotate)
    set = _swig_new_instance_method(_fifechan.Point_set)
    __swig_destroy__ = _fifechan.delete_Point

# Register Point in _fifechan:
_fifechan.Point_swigregister(Point)
class Widget(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    Selection_None = _fifechan.Widget_Selection_None
    Selection_Border = _fifechan.Widget_Selection_Border
    Selection_Background = _fifechan.Widget_Selection_Background
    getParent = _swig_new_instance_method(_fifechan.Widget_getParent)
    setWidth = _swig_new_instance_method(_fifechan.Widget_setWidth)
    getWidth = _swig_new_instance_method(_fifechan.Widget_getWidth)
    setHeight = _swig_new_instance_method(_fifechan.Widget_setHeight)
    getHeight = _swig_new_instance_method(_fifechan.Widget_getHeight)
    setSize = _swig_new_instance_method(_fifechan.Widget_setSize)
    setX = _swig_new_instance_method(_fifechan.Widget_setX)
    getX = _swig_new_instance_method(_fifechan.Widget_getX)
    setY = _swig_new_instance_method(_fifechan.Widget_setY)
    getY = _swig_new_instance_method(_fifechan.Widget_getY)
    setPosition = _swig_new_instance_method(_fifechan.Widget_setPosition)
    setOutlineSize = _swig_new_instance_method(_fifechan.Widget_setOutlineSize)
    getOutlineSize = _swig_new_instance_method(_fifechan.Widget_getOutlineSize)
    setBorderSize = _swig_new_instance_method(_fifechan.Widget_setBorderSize)
    getBorderSize = _swig_new_instance_method(_fifechan.Widget_getBorderSize)
    setMargin = _swig_new_instance_method(_fifechan.Widget_setMargin)
    setMarginTop = _swig_new_instance_method(_fifechan.Widget_setMarginTop)
    getMarginTop = _swig_new_instance_method(_fifechan.Widget_getMarginTop)
    setMarginRight = _swig_new_instance_method(_fifechan.Widget_setMarginRight)
    getMarginRight = _swig_new_instance_method(_fifechan.Widget_getMarginRight)
    setMarginBottom = _swig_new_instance_method(_fifechan.Widget_setMarginBottom)
    getMarginBottom = _swig_new_instance_method(_fifechan.Widget_getMarginBottom)
    setMarginLeft = _swig_new_instance_method(_fifechan.Widget_setMarginLeft)
    getMarginLeft = _swig_new_instance_method(_fifechan.Widget_getMarginLeft)
    setPadding = _swig_new_instance_method(_fifechan.Widget_setPadding)
    setPaddingTop = _swig_new_instance_method(_fifechan.Widget_setPaddingTop)
    getPaddingTop = _swig_new_instance_method(_fifechan.Widget_getPaddingTop)
    setPaddingRight = _swig_new_instance_method(_fifechan.Widget_setPaddingRight)
    getPaddingRight = _swig_new_instance_method(_fifechan.Widget_getPaddingRight)
    setPaddingBottom = _swig_new_instance_method(_fifechan.Widget_setPaddingBottom)
    getPaddingBottom = _swig_new_instance_method(_fifechan.Widget_getPaddingBottom)
    setPaddingLeft = _swig_new_instance_method(_fifechan.Widget_setPaddingLeft)
    getPaddingLeft = _swig_new_instance_method(_fifechan.Widget_getPaddingLeft)
    setFocusable = _swig_new_instance_method(_fifechan.Widget_setFocusable)
    isFocusable = _swig_new_instance_method(_fifechan.Widget_isFocusable)
    isFocused = _swig_new_instance_method(_fifechan.Widget_isFocused)
    setEnabled = _swig_new_instance_method(_fifechan.Widget_setEnabled)
    isEnabled = _swig_new_instance_method(_fifechan.Widget_isEnabled)
    setVisible = _swig_new_instance_method(_fifechan.Widget_setVisible)
    isVisible = _swig_new_instance_method(_fifechan.Widget_isVisible)
    isSetVisible = _swig_new_instance_method(_fifechan.Widget_isSetVisible)
    setBaseColor = _swig_new_instance_method(_fifechan.Widget_setBaseColor)
    getBaseColor = _swig_new_instance_method(_fifechan.Widget_getBaseColor)
    setForegroundColor = _swig_new_instance_method(_fifechan.Widget_setForegroundColor)
    getForegroundColor = _swig_new_instance_method(_fifechan.Widget_getForegroundColor)
    setBackgroundColor = _swig_new_instance_method(_fifechan.Widget_setBackgroundColor)
    getBackgroundColor = _swig_new_instance_method(_fifechan.Widget_getBackgroundColor)
    setSelectionColor = _swig_new_instance_method(_fifechan.Widget_setSelectionColor)
    getSelectionColor = _swig_new_instance_method(_fifechan.Widget_getSelectionColor)
    setOutlineColor = _swig_new_instance_method(_fifechan.Widget_setOutlineColor)
    getOutlineColor = _swig_new_instance_method(_fifechan.Widget_getOutlineColor)
    setBorderColor = _swig_new_instance_method(_fifechan.Widget_setBorderColor)
    getBorderColor = _swig_new_instance_method(_fifechan.Widget_getBorderColor)
    setSelectionMode = _swig_new_instance_method(_fifechan.Widget_setSelectionMode)
    getSelectionMode = _swig_new_instance_method(_fifechan.Widget_getSelectionMode)
    setMinSize = _swig_new_instance_method(_fifechan.Widget_setMinSize)
    getMinSize = _swig_new_instance_method(_fifechan.Widget_getMinSize)
    setMaxSize = _swig_new_instance_method(_fifechan.Widget_setMaxSize)
    getMaxSize = _swig_new_instance_method(_fifechan.Widget_getMaxSize)
    setFixedSize = _swig_new_instance_method(_fifechan.Widget_setFixedSize)
    getFixedSize = _swig_new_instance_method(_fifechan.Widget_getFixedSize)
    isFixedSize = _swig_new_instance_method(_fifechan.Widget_isFixedSize)
    setVerticalExpand = _swig_new_instance_method(_fifechan.Widget_setVerticalExpand)
    isVerticalExpand = _swig_new_instance_method(_fifechan.Widget_isVerticalExpand)
    setHorizontalExpand = _swig_new_instance_method(_fifechan.Widget_setHorizontalExpand)
    isHorizontalExpand = _swig_new_instance_method(_fifechan.Widget_isHorizontalExpand)
    isLayouted = _swig_new_instance_method(_fifechan.Widget_isLayouted)
    adaptLayout = _swig_new_instance_method(_fifechan.Widget_adaptLayout)
    resizeToContent = _swig_new_instance_method(_fifechan.Widget_resizeToContent)
    adjustSize = _swig_new_instance_method(_fifechan.Widget_adjustSize)
    expandContent = _swig_new_instance_method(_fifechan.Widget_expandContent)
    requestFocus = _swig_new_instance_method(_fifechan.Widget_requestFocus)
    requestMoveToTop = _swig_new_instance_method(_fifechan.Widget_requestMoveToTop)
    requestMoveToBottom = _swig_new_instance_method(_fifechan.Widget_requestMoveToBottom)
    setActionEventId = _swig_new_instance_method(_fifechan.Widget_setActionEventId)
    getActionEventId = _swig_new_instance_method(_fifechan.Widget_getActionEventId)
    getAbsolutePosition = _swig_new_instance_method(_fifechan.Widget_getAbsolutePosition)
    getFont = _swig_new_instance_method(_fifechan.Widget_getFont)
    setGlobalFont = _swig_new_static_method(_fifechan.Widget_setGlobalFont)
    setFont = _swig_new_instance_method(_fifechan.Widget_setFont)
    isTabInEnabled = _swig_new_instance_method(_fifechan.Widget_isTabInEnabled)
    setTabInEnabled = _swig_new_instance_method(_fifechan.Widget_setTabInEnabled)
    isTabOutEnabled = _swig_new_instance_method(_fifechan.Widget_isTabOutEnabled)
    setTabOutEnabled = _swig_new_instance_method(_fifechan.Widget_setTabOutEnabled)
    isModalFocusable = _swig_new_instance_method(_fifechan.Widget_isModalFocusable)
    isModalMouseInputFocusable = _swig_new_instance_method(_fifechan.Widget_isModalMouseInputFocusable)
    requestModalFocus = _swig_new_instance_method(_fifechan.Widget_requestModalFocus)
    requestModalMouseInputFocus = _swig_new_instance_method(_fifechan.Widget_requestModalMouseInputFocus)
    releaseModalFocus = _swig_new_instance_method(_fifechan.Widget_releaseModalFocus)
    releaseModalMouseInputFocus = _swig_new_instance_method(_fifechan.Widget_releaseModalMouseInputFocus)
    isModalFocused = _swig_new_instance_method(_fifechan.Widget_isModalFocused)
    isModalMouseInputFocused = _swig_new_instance_method(_fifechan.Widget_isModalMouseInputFocused)
    getWidgetAt = _swig_new_instance_method(_fifechan.Widget_getWidgetAt)
    moveToTop = _swig_new_instance_method(_fifechan.Widget_moveToTop)
    moveToBottom = _swig_new_instance_method(_fifechan.Widget_moveToBottom)
    focusNext = _swig_new_instance_method(_fifechan.Widget_focusNext)
    focusPrevious = _swig_new_instance_method(_fifechan.Widget_focusPrevious)
    addActionListener = _swig_new_instance_method(_fifechan.Widget_addActionListener)
    removeActionListener = _swig_new_instance_method(_fifechan.Widget_removeActionListener)
    addMouseListener = _swig_new_instance_method(_fifechan.Widget_addMouseListener)
    removeMouseListener = _swig_new_instance_method(_fifechan.Widget_removeMouseListener)
    addKeyListener = _swig_new_instance_method(_fifechan.Widget_addKeyListener)
    removeKeyListener = _swig_new_instance_method(_fifechan.Widget_removeKeyListener)
    addWidgetListener = _swig_new_instance_method(_fifechan.Widget_addWidgetListener)
    removeWidgetListener = _swig_new_instance_method(_fifechan.Widget_removeWidgetListener)
    draw = _swig_new_instance_method(_fifechan.Widget_draw)
    __swig_destroy__ = _fifechan.delete_Widget

# Register Widget in _fifechan:
_fifechan.Widget_swigregister(Widget)
class Spacer(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _fifechan.Spacer_swiginit(self, _fifechan.new_Spacer())
    __swig_destroy__ = _fifechan.delete_Spacer
    resizeToContent = _swig_new_instance_method(_fifechan.Spacer_resizeToContent)

# Register Spacer in _fifechan:
_fifechan.Spacer_swigregister(Spacer)
class Container(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _fifechan.Container_swiginit(self, _fifechan.new_Container())
    __swig_destroy__ = _fifechan.delete_Container
    setOpaque = _swig_new_instance_method(_fifechan.Container_setOpaque)
    isOpaque = _swig_new_instance_method(_fifechan.Container_isOpaque)
    add = _swig_new_instance_method(_fifechan.Container_add)
    remove = _swig_new_instance_method(_fifechan.Container_remove)
    clear = _swig_new_instance_method(_fifechan.Container_clear)
    Absolute = _fifechan.Container_Absolute
    AutoSize = _fifechan.Container_AutoSize
    Vertical = _fifechan.Container_Vertical
    Horizontal = _fifechan.Container_Horizontal
    Circular = _fifechan.Container_Circular
    setLayout = _swig_new_instance_method(_fifechan.Container_setLayout)
    getLayout = _swig_new_instance_method(_fifechan.Container_getLayout)
    setUniformSize = _swig_new_instance_method(_fifechan.Container_setUniformSize)
    isUniformSize = _swig_new_instance_method(_fifechan.Container_isUniformSize)
    setVerticalSpacing = _swig_new_instance_method(_fifechan.Container_setVerticalSpacing)
    getVerticalSpacing = _swig_new_instance_method(_fifechan.Container_getVerticalSpacing)
    setHorizontalSpacing = _swig_new_instance_method(_fifechan.Container_setHorizontalSpacing)
    getHorizontalSpacing = _swig_new_instance_method(_fifechan.Container_getHorizontalSpacing)
    setBackgroundWidget = _swig_new_instance_method(_fifechan.Container_setBackgroundWidget)
    getBackgroundWidget = _swig_new_instance_method(_fifechan.Container_getBackgroundWidget)

# Register Container in _fifechan:
_fifechan.Container_swigregister(Container)
class Button(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.Button_swiginit(self, _fifechan.new_Button(*args))
    setCaption = _swig_new_instance_method(_fifechan.Button_setCaption)
    getCaption = _swig_new_instance_method(_fifechan.Button_getCaption)
    setActive = _swig_new_instance_method(_fifechan.Button_setActive)
    isActive = _swig_new_instance_method(_fifechan.Button_isActive)
    setAlignment = _swig_new_instance_method(_fifechan.Button_setAlignment)
    getAlignment = _swig_new_instance_method(_fifechan.Button_getAlignment)
    setDownXOffset = _swig_new_instance_method(_fifechan.Button_setDownXOffset)
    getDownXOffset = _swig_new_instance_method(_fifechan.Button_getDownXOffset)
    setDownYOffset = _swig_new_instance_method(_fifechan.Button_setDownYOffset)
    getDownYOffset = _swig_new_instance_method(_fifechan.Button_getDownYOffset)
    setDownOffset = _swig_new_instance_method(_fifechan.Button_setDownOffset)
    adjustSize = _swig_new_instance_method(_fifechan.Button_adjustSize)
    __swig_destroy__ = _fifechan.delete_Button

# Register Button in _fifechan:
_fifechan.Button_swigregister(Button)
class ImageButton(Button):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.ImageButton_swiginit(self, _fifechan.new_ImageButton(*args))
    __swig_destroy__ = _fifechan.delete_ImageButton
    setUpImage = _swig_new_instance_method(_fifechan.ImageButton_setUpImage)
    getUpImage = _swig_new_instance_method(_fifechan.ImageButton_getUpImage)
    setDownImage = _swig_new_instance_method(_fifechan.ImageButton_setDownImage)
    getDownImage = _swig_new_instance_method(_fifechan.ImageButton_getDownImage)
    setHoverImage = _swig_new_instance_method(_fifechan.ImageButton_setHoverImage)
    getHoverImage = _swig_new_instance_method(_fifechan.ImageButton_getHoverImage)
    setInactiveUpImage = _swig_new_instance_method(_fifechan.ImageButton_setInactiveUpImage)
    getInactiveUpImage = _swig_new_instance_method(_fifechan.ImageButton_getInactiveUpImage)
    setInactiveDownImage = _swig_new_instance_method(_fifechan.ImageButton_setInactiveDownImage)
    getInactiveDownImage = _swig_new_instance_method(_fifechan.ImageButton_getInactiveDownImage)
    setInactiveHoverImage = _swig_new_instance_method(_fifechan.ImageButton_setInactiveHoverImage)
    getInactiveHoverImage = _swig_new_instance_method(_fifechan.ImageButton_getInactiveHoverImage)
    resizeToContent = _swig_new_instance_method(_fifechan.ImageButton_resizeToContent)
    adjustSize = _swig_new_instance_method(_fifechan.ImageButton_adjustSize)

# Register ImageButton in _fifechan:
_fifechan.ImageButton_swigregister(ImageButton)
class ToggleButton(ImageButton):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.ToggleButton_swiginit(self, _fifechan.new_ToggleButton(*args))
    __swig_destroy__ = _fifechan.delete_ToggleButton
    isSelected = _swig_new_instance_method(_fifechan.ToggleButton_isSelected)
    setSelected = _swig_new_instance_method(_fifechan.ToggleButton_setSelected)
    toggleSelected = _swig_new_instance_method(_fifechan.ToggleButton_toggleSelected)
    setGroup = _swig_new_instance_method(_fifechan.ToggleButton_setGroup)
    getGroup = _swig_new_instance_method(_fifechan.ToggleButton_getGroup)

# Register ToggleButton in _fifechan:
_fifechan.ToggleButton_swigregister(ToggleButton)
class CheckBox(ImageButton):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    Marker_Checkmark = _fifechan.CheckBox_Marker_Checkmark
    Marker_Cross = _fifechan.CheckBox_Marker_Cross
    Marker_Dot = _fifechan.CheckBox_Marker_Dot
    Marker_Rhombus = _fifechan.CheckBox_Marker_Rhombus
    Marker_Image = _fifechan.CheckBox_Marker_Image

    def __init__(self):
        _fifechan.CheckBox_swiginit(self, _fifechan.new_CheckBox())
    __swig_destroy__ = _fifechan.delete_CheckBox
    isSelected = _swig_new_instance_method(_fifechan.CheckBox_isSelected)
    setSelected = _swig_new_instance_method(_fifechan.CheckBox_setSelected)
    toggleSelected = _swig_new_instance_method(_fifechan.CheckBox_toggleSelected)
    setBackgroundImage = _swig_new_instance_method(_fifechan.CheckBox_setBackgroundImage)
    getBackgroundImage = _swig_new_instance_method(_fifechan.CheckBox_getBackgroundImage)
    getMarkerStyle = _swig_new_instance_method(_fifechan.CheckBox_getMarkerStyle)
    setMarkerStyle = _swig_new_instance_method(_fifechan.CheckBox_setMarkerStyle)
    adjustSize = _swig_new_instance_method(_fifechan.CheckBox_adjustSize)

# Register CheckBox in _fifechan:
_fifechan.CheckBox_swigregister(CheckBox)
class Icon(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, image):
        _fifechan.Icon_swiginit(self, _fifechan.new_Icon(image))
    __swig_destroy__ = _fifechan.delete_Icon
    setImage = _swig_new_instance_method(_fifechan.Icon_setImage)
    getImage = _swig_new_instance_method(_fifechan.Icon_getImage)
    isScaling = _swig_new_instance_method(_fifechan.Icon_isScaling)
    setScaling = _swig_new_instance_method(_fifechan.Icon_setScaling)
    isTiling = _swig_new_instance_method(_fifechan.Icon_isTiling)
    setTiling = _swig_new_instance_method(_fifechan.Icon_setTiling)
    setOpaque = _swig_new_instance_method(_fifechan.Icon_setOpaque)
    isOpaque = _swig_new_instance_method(_fifechan.Icon_isOpaque)
    adjustSize = _swig_new_instance_method(_fifechan.Icon_adjustSize)

# Register Icon in _fifechan:
_fifechan.Icon_swigregister(Icon)
class ScrollArea(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.ScrollArea_swiginit(self, _fifechan.new_ScrollArea(*args))
    __swig_destroy__ = _fifechan.delete_ScrollArea
    setContent = _swig_new_instance_method(_fifechan.ScrollArea_setContent)
    getContent = _swig_new_instance_method(_fifechan.ScrollArea_getContent)
    setHorizontalScrollPolicy = _swig_new_instance_method(_fifechan.ScrollArea_setHorizontalScrollPolicy)
    getHorizontalScrollPolicy = _swig_new_instance_method(_fifechan.ScrollArea_getHorizontalScrollPolicy)
    setVerticalScrollPolicy = _swig_new_instance_method(_fifechan.ScrollArea_setVerticalScrollPolicy)
    getVerticalScrollPolicy = _swig_new_instance_method(_fifechan.ScrollArea_getVerticalScrollPolicy)
    setScrollPolicy = _swig_new_instance_method(_fifechan.ScrollArea_setScrollPolicy)
    setVerticalScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_setVerticalScrollAmount)
    getVerticalScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_getVerticalScrollAmount)
    setHorizontalScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_setHorizontalScrollAmount)
    getHorizontalScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_getHorizontalScrollAmount)
    setScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_setScrollAmount)
    getHorizontalMaxScroll = _swig_new_instance_method(_fifechan.ScrollArea_getHorizontalMaxScroll)
    getVerticalMaxScroll = _swig_new_instance_method(_fifechan.ScrollArea_getVerticalMaxScroll)
    setScrollbarWidth = _swig_new_instance_method(_fifechan.ScrollArea_setScrollbarWidth)
    getScrollbarWidth = _swig_new_instance_method(_fifechan.ScrollArea_getScrollbarWidth)
    setLeftButtonScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_setLeftButtonScrollAmount)
    setRightButtonScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_setRightButtonScrollAmount)
    setUpButtonScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_setUpButtonScrollAmount)
    setDownButtonScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_setDownButtonScrollAmount)
    getLeftButtonScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_getLeftButtonScrollAmount)
    getRightButtonScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_getRightButtonScrollAmount)
    getUpButtonScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_getUpButtonScrollAmount)
    getDownButtonScrollAmount = _swig_new_instance_method(_fifechan.ScrollArea_getDownButtonScrollAmount)
    ShowAlways = _fifechan.ScrollArea_ShowAlways
    ShowNever = _fifechan.ScrollArea_ShowNever
    ShowAuto = _fifechan.ScrollArea_ShowAuto

# Register ScrollArea in _fifechan:
_fifechan.ScrollArea_swigregister(ScrollArea)
class ListModel(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    __swig_destroy__ = _fifechan.delete_ListModel
    getNumberOfElements = _swig_new_instance_method(_fifechan.ListModel_getNumberOfElements)
    getElementAt = _swig_new_instance_method(_fifechan.ListModel_getElementAt)

    def __init__(self):
        if self.__class__ == ListModel:
            _self = None
        else:
            _self = self
        _fifechan.ListModel_swiginit(self, _fifechan.new_ListModel(_self, ))
    def __disown__(self):
        self.this.disown()
        _fifechan.disown_ListModel(self)
        return weakref.proxy(self)

# Register ListModel in _fifechan:
_fifechan.ListModel_swigregister(ListModel)
class ListBox(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.ListBox_swiginit(self, _fifechan.new_ListBox(*args))
    __swig_destroy__ = _fifechan.delete_ListBox
    getSelected = _swig_new_instance_method(_fifechan.ListBox_getSelected)
    setSelected = _swig_new_instance_method(_fifechan.ListBox_setSelected)
    setListModel = _swig_new_instance_method(_fifechan.ListBox_setListModel)
    getListModel = _swig_new_instance_method(_fifechan.ListBox_getListModel)
    adjustSize = _swig_new_instance_method(_fifechan.ListBox_adjustSize)
    isWrappingEnabled = _swig_new_instance_method(_fifechan.ListBox_isWrappingEnabled)
    setWrappingEnabled = _swig_new_instance_method(_fifechan.ListBox_setWrappingEnabled)

# Register ListBox in _fifechan:
_fifechan.ListBox_swigregister(ListBox)
class DropDown(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, listModel=None, scrollArea=None, listBox=None):
        _fifechan.DropDown_swiginit(self, _fifechan.new_DropDown(listModel, scrollArea, listBox))
    __swig_destroy__ = _fifechan.delete_DropDown
    getSelected = _swig_new_instance_method(_fifechan.DropDown_getSelected)
    setSelected = _swig_new_instance_method(_fifechan.DropDown_setSelected)
    setListModel = _swig_new_instance_method(_fifechan.DropDown_setListModel)
    getListModel = _swig_new_instance_method(_fifechan.DropDown_getListModel)
    adjustHeight = _swig_new_instance_method(_fifechan.DropDown_adjustHeight)
    setBaseColor = _swig_new_instance_method(_fifechan.DropDown_setBaseColor)
    getBaseColor = _swig_new_instance_method(_fifechan.DropDown_getBaseColor)
    setForegroundColor = _swig_new_instance_method(_fifechan.DropDown_setForegroundColor)
    getForegroundColor = _swig_new_instance_method(_fifechan.DropDown_getForegroundColor)
    setBackgroundColor = _swig_new_instance_method(_fifechan.DropDown_setBackgroundColor)
    getBackgroundColor = _swig_new_instance_method(_fifechan.DropDown_getBackgroundColor)
    setSelectionColor = _swig_new_instance_method(_fifechan.DropDown_setSelectionColor)
    getSelectionColor = _swig_new_instance_method(_fifechan.DropDown_getSelectionColor)

# Register DropDown in _fifechan:
_fifechan.DropDown_swigregister(DropDown)
class RadioButton(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.RadioButton_swiginit(self, _fifechan.new_RadioButton(*args))
    __swig_destroy__ = _fifechan.delete_RadioButton
    isSelected = _swig_new_instance_method(_fifechan.RadioButton_isSelected)
    setSelected = _swig_new_instance_method(_fifechan.RadioButton_setSelected)
    getCaption = _swig_new_instance_method(_fifechan.RadioButton_getCaption)
    setCaption = _swig_new_instance_method(_fifechan.RadioButton_setCaption)
    setGroup = _swig_new_instance_method(_fifechan.RadioButton_setGroup)
    getGroup = _swig_new_instance_method(_fifechan.RadioButton_getGroup)
    adjustSize = _swig_new_instance_method(_fifechan.RadioButton_adjustSize)

# Register RadioButton in _fifechan:
_fifechan.RadioButton_swigregister(RadioButton)
class Slider(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.Slider_swiginit(self, _fifechan.new_Slider(*args))
    __swig_destroy__ = _fifechan.delete_Slider
    setScale = _swig_new_instance_method(_fifechan.Slider_setScale)
    getScaleStart = _swig_new_instance_method(_fifechan.Slider_getScaleStart)
    setScaleStart = _swig_new_instance_method(_fifechan.Slider_setScaleStart)
    getScaleEnd = _swig_new_instance_method(_fifechan.Slider_getScaleEnd)
    setScaleEnd = _swig_new_instance_method(_fifechan.Slider_setScaleEnd)
    getValue = _swig_new_instance_method(_fifechan.Slider_getValue)
    setValue = _swig_new_instance_method(_fifechan.Slider_setValue)
    setMarkerLength = _swig_new_instance_method(_fifechan.Slider_setMarkerLength)
    getMarkerLength = _swig_new_instance_method(_fifechan.Slider_getMarkerLength)
    setOrientation = _swig_new_instance_method(_fifechan.Slider_setOrientation)
    getOrientation = _swig_new_instance_method(_fifechan.Slider_getOrientation)
    setStepLength = _swig_new_instance_method(_fifechan.Slider_setStepLength)
    getStepLength = _swig_new_instance_method(_fifechan.Slider_getStepLength)
    Horizontal = _fifechan.Slider_Horizontal
    Vertical = _fifechan.Slider_Vertical

# Register Slider in _fifechan:
_fifechan.Slider_swigregister(Slider)
class Window(Container):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.Window_swiginit(self, _fifechan.new_Window(*args))
    __swig_destroy__ = _fifechan.delete_Window
    setCaption = _swig_new_instance_method(_fifechan.Window_setCaption)
    getCaption = _swig_new_instance_method(_fifechan.Window_getCaption)
    setAlignment = _swig_new_instance_method(_fifechan.Window_setAlignment)
    getAlignment = _swig_new_instance_method(_fifechan.Window_getAlignment)
    setTitleBarHeight = _swig_new_instance_method(_fifechan.Window_setTitleBarHeight)
    getTitleBarHeight = _swig_new_instance_method(_fifechan.Window_getTitleBarHeight)
    setMovable = _swig_new_instance_method(_fifechan.Window_setMovable)
    isMovable = _swig_new_instance_method(_fifechan.Window_isMovable)
    setOpaque = _swig_new_instance_method(_fifechan.Window_setOpaque)
    isOpaque = _swig_new_instance_method(_fifechan.Window_isOpaque)
    resizeToContent = _swig_new_instance_method(_fifechan.Window_resizeToContent)

# Register Window in _fifechan:
_fifechan.Window_swigregister(Window)
class AdjustingContainer(Container):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _fifechan.AdjustingContainer_swiginit(self, _fifechan.new_AdjustingContainer())
    __swig_destroy__ = _fifechan.delete_AdjustingContainer
    setNumberOfColumns = _swig_new_instance_method(_fifechan.AdjustingContainer_setNumberOfColumns)
    getNumberOfColumns = _swig_new_instance_method(_fifechan.AdjustingContainer_getNumberOfColumns)
    setColumnAlignment = _swig_new_instance_method(_fifechan.AdjustingContainer_setColumnAlignment)
    getColumnAlignment = _swig_new_instance_method(_fifechan.AdjustingContainer_getColumnAlignment)
    adjustContent = _swig_new_instance_method(_fifechan.AdjustingContainer_adjustContent)
    LEFT = _fifechan.AdjustingContainer_LEFT
    CENTER = _fifechan.AdjustingContainer_CENTER
    RIGHT = _fifechan.AdjustingContainer_RIGHT

# Register AdjustingContainer in _fifechan:
_fifechan.AdjustingContainer_swigregister(AdjustingContainer)
class FlowContainer(Container):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _fifechan.FlowContainer_swiginit(self, _fifechan.new_FlowContainer())
    __swig_destroy__ = _fifechan.delete_FlowContainer
    setAlignment = _swig_new_instance_method(_fifechan.FlowContainer_setAlignment)
    getAlignment = _swig_new_instance_method(_fifechan.FlowContainer_getAlignment)
    adjustContent = _swig_new_instance_method(_fifechan.FlowContainer_adjustContent)
    Left = _fifechan.FlowContainer_Left
    Right = _fifechan.FlowContainer_Right
    Top = _fifechan.FlowContainer_Top
    Bottom = _fifechan.FlowContainer_Bottom
    Center = _fifechan.FlowContainer_Center

# Register FlowContainer in _fifechan:
_fifechan.FlowContainer_swigregister(FlowContainer)
class TextBox(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.TextBox_swiginit(self, _fifechan.new_TextBox(*args))
    setText = _swig_new_instance_method(_fifechan.TextBox_setText)
    getText = _swig_new_instance_method(_fifechan.TextBox_getText)
    getTextRow = _swig_new_instance_method(_fifechan.TextBox_getTextRow)
    setTextRow = _swig_new_instance_method(_fifechan.TextBox_setTextRow)
    getNumberOfRows = _swig_new_instance_method(_fifechan.TextBox_getNumberOfRows)
    getCaretPosition = _swig_new_instance_method(_fifechan.TextBox_getCaretPosition)
    setCaretPosition = _swig_new_instance_method(_fifechan.TextBox_setCaretPosition)
    getCaretRow = _swig_new_instance_method(_fifechan.TextBox_getCaretRow)
    setCaretRow = _swig_new_instance_method(_fifechan.TextBox_setCaretRow)
    getCaretColumn = _swig_new_instance_method(_fifechan.TextBox_getCaretColumn)
    setCaretColumn = _swig_new_instance_method(_fifechan.TextBox_setCaretColumn)
    setCaretRowColumn = _swig_new_instance_method(_fifechan.TextBox_setCaretRowColumn)
    scrollToCaret = _swig_new_instance_method(_fifechan.TextBox_scrollToCaret)
    isEditable = _swig_new_instance_method(_fifechan.TextBox_isEditable)
    setEditable = _swig_new_instance_method(_fifechan.TextBox_setEditable)
    addRow = _swig_new_instance_method(_fifechan.TextBox_addRow)
    isOpaque = _swig_new_instance_method(_fifechan.TextBox_isOpaque)
    setOpaque = _swig_new_instance_method(_fifechan.TextBox_setOpaque)
    __swig_destroy__ = _fifechan.delete_TextBox

# Register TextBox in _fifechan:
_fifechan.TextBox_swigregister(TextBox)
class TextField(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.TextField_swiginit(self, _fifechan.new_TextField(*args))
    __swig_destroy__ = _fifechan.delete_TextField
    setText = _swig_new_instance_method(_fifechan.TextField_setText)
    getText = _swig_new_instance_method(_fifechan.TextField_getText)
    adjustSize = _swig_new_instance_method(_fifechan.TextField_adjustSize)
    adjustHeight = _swig_new_instance_method(_fifechan.TextField_adjustHeight)
    setCaretPosition = _swig_new_instance_method(_fifechan.TextField_setCaretPosition)
    getCaretPosition = _swig_new_instance_method(_fifechan.TextField_getCaretPosition)

# Register TextField in _fifechan:
_fifechan.TextField_swigregister(TextField)
class PasswordField(TextField):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args, **kwargs):
        _fifechan.PasswordField_swiginit(self, _fifechan.new_PasswordField(*args, **kwargs))
    __swig_destroy__ = _fifechan.delete_PasswordField

# Register PasswordField in _fifechan:
_fifechan.PasswordField_swigregister(PasswordField)
class IconProgressBar(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.IconProgressBar_swiginit(self, _fifechan.new_IconProgressBar(*args))
    __swig_destroy__ = _fifechan.delete_IconProgressBar
    setImage = _swig_new_instance_method(_fifechan.IconProgressBar_setImage)
    getImage = _swig_new_instance_method(_fifechan.IconProgressBar_getImage)
    setMaxIcons = _swig_new_instance_method(_fifechan.IconProgressBar_setMaxIcons)
    getMaxIcons = _swig_new_instance_method(_fifechan.IconProgressBar_getMaxIcons)
    setOrientation = _swig_new_instance_method(_fifechan.IconProgressBar_setOrientation)
    getOrientation = _swig_new_instance_method(_fifechan.IconProgressBar_getOrientation)
    advance = _swig_new_instance_method(_fifechan.IconProgressBar_advance)
    reset = _swig_new_instance_method(_fifechan.IconProgressBar_reset)
    setIconCount = _swig_new_instance_method(_fifechan.IconProgressBar_setIconCount)
    getIconCount = _swig_new_instance_method(_fifechan.IconProgressBar_getIconCount)
    setOpaque = _swig_new_instance_method(_fifechan.IconProgressBar_setOpaque)
    isOpaque = _swig_new_instance_method(_fifechan.IconProgressBar_isOpaque)
    HORIZONTAL = _fifechan.IconProgressBar_HORIZONTAL
    VERTICAL = _fifechan.IconProgressBar_VERTICAL

# Register IconProgressBar in _fifechan:
_fifechan.IconProgressBar_swigregister(IconProgressBar)
class ImageProgressBar(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.ImageProgressBar_swiginit(self, _fifechan.new_ImageProgressBar(*args))
    __swig_destroy__ = _fifechan.delete_ImageProgressBar
    setBarImage = _swig_new_instance_method(_fifechan.ImageProgressBar_setBarImage)
    getBarImage = _swig_new_instance_method(_fifechan.ImageProgressBar_getBarImage)
    setForegroundImage = _swig_new_instance_method(_fifechan.ImageProgressBar_setForegroundImage)
    getForegroundImage = _swig_new_instance_method(_fifechan.ImageProgressBar_getForegroundImage)
    setMaxValue = _swig_new_instance_method(_fifechan.ImageProgressBar_setMaxValue)
    getMaxValue = _swig_new_instance_method(_fifechan.ImageProgressBar_getMaxValue)
    setValue = _swig_new_instance_method(_fifechan.ImageProgressBar_setValue)
    getValue = _swig_new_instance_method(_fifechan.ImageProgressBar_getValue)
    setOrientation = _swig_new_instance_method(_fifechan.ImageProgressBar_setOrientation)
    getOrientation = _swig_new_instance_method(_fifechan.ImageProgressBar_getOrientation)
    setOpaque = _swig_new_instance_method(_fifechan.ImageProgressBar_setOpaque)
    isOpaque = _swig_new_instance_method(_fifechan.ImageProgressBar_isOpaque)
    HORIZONTAL = _fifechan.ImageProgressBar_HORIZONTAL
    VERTICAL = _fifechan.ImageProgressBar_VERTICAL

# Register ImageProgressBar in _fifechan:
_fifechan.ImageProgressBar_swigregister(ImageProgressBar)
class Tab(Container):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _fifechan.Tab_swiginit(self, _fifechan.new_Tab())
    __swig_destroy__ = _fifechan.delete_Tab
    setTabbedArea = _swig_new_instance_method(_fifechan.Tab_setTabbedArea)
    getTabbedArea = _swig_new_instance_method(_fifechan.Tab_getTabbedArea)

# Register Tab in _fifechan:
_fifechan.Tab_swigregister(Tab)
class TabbedArea(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _fifechan.TabbedArea_swiginit(self, _fifechan.new_TabbedArea())
    __swig_destroy__ = _fifechan.delete_TabbedArea
    setOpaque = _swig_new_instance_method(_fifechan.TabbedArea_setOpaque)
    isOpaque = _swig_new_instance_method(_fifechan.TabbedArea_isOpaque)
    addTab = _swig_new_instance_method(_fifechan.TabbedArea_addTab)
    removeTabWithIndex = _swig_new_instance_method(_fifechan.TabbedArea_removeTabWithIndex)
    removeTab = _swig_new_instance_method(_fifechan.TabbedArea_removeTab)
    getNumberOfTabs = _swig_new_instance_method(_fifechan.TabbedArea_getNumberOfTabs)
    isTabSelected = _swig_new_instance_method(_fifechan.TabbedArea_isTabSelected)
    setSelectedTab = _swig_new_instance_method(_fifechan.TabbedArea_setSelectedTab)
    getSelectedTabIndex = _swig_new_instance_method(_fifechan.TabbedArea_getSelectedTabIndex)
    getSelectedTab = _swig_new_instance_method(_fifechan.TabbedArea_getSelectedTab)
    setBaseColor = _swig_new_instance_method(_fifechan.TabbedArea_setBaseColor)
    setBackgroundWidget = _swig_new_instance_method(_fifechan.TabbedArea_setBackgroundWidget)
    getBackgroundWidget = _swig_new_instance_method(_fifechan.TabbedArea_getBackgroundWidget)
    setLayout = _swig_new_instance_method(_fifechan.TabbedArea_setLayout)
    getLayout = _swig_new_instance_method(_fifechan.TabbedArea_getLayout)
    setUniformSize = _swig_new_instance_method(_fifechan.TabbedArea_setUniformSize)
    isUniformSize = _swig_new_instance_method(_fifechan.TabbedArea_isUniformSize)
    setVerticalSpacing = _swig_new_instance_method(_fifechan.TabbedArea_setVerticalSpacing)
    getVerticalSpacing = _swig_new_instance_method(_fifechan.TabbedArea_getVerticalSpacing)
    setHorizontalSpacing = _swig_new_instance_method(_fifechan.TabbedArea_setHorizontalSpacing)
    getHorizontalSpacing = _swig_new_instance_method(_fifechan.TabbedArea_getHorizontalSpacing)

# Register TabbedArea in _fifechan:
_fifechan.TabbedArea_swigregister(TabbedArea)
class BarGraph(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.BarGraph_swiginit(self, _fifechan.new_BarGraph(*args))
    __swig_destroy__ = _fifechan.delete_BarGraph
    setBarX = _swig_new_instance_method(_fifechan.BarGraph_setBarX)
    getBarX = _swig_new_instance_method(_fifechan.BarGraph_getBarX)
    setBarY = _swig_new_instance_method(_fifechan.BarGraph_setBarY)
    getBarY = _swig_new_instance_method(_fifechan.BarGraph_getBarY)
    setBarPosition = _swig_new_instance_method(_fifechan.BarGraph_setBarPosition)
    setBarWidth = _swig_new_instance_method(_fifechan.BarGraph_setBarWidth)
    getBarWidth = _swig_new_instance_method(_fifechan.BarGraph_getBarWidth)
    setBarHeight = _swig_new_instance_method(_fifechan.BarGraph_setBarHeight)
    getBarHeight = _swig_new_instance_method(_fifechan.BarGraph_getBarHeight)
    setBarSize = _swig_new_instance_method(_fifechan.BarGraph_setBarSize)
    setOpaque = _swig_new_instance_method(_fifechan.BarGraph_setOpaque)
    isOpaque = _swig_new_instance_method(_fifechan.BarGraph_isOpaque)

# Register BarGraph in _fifechan:
_fifechan.BarGraph_swigregister(BarGraph)
class CurveGraph(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.CurveGraph_swiginit(self, _fifechan.new_CurveGraph(*args))
    __swig_destroy__ = _fifechan.delete_CurveGraph
    setPointVector = _swig_new_instance_method(_fifechan.CurveGraph_setPointVector)
    getPointVector = _swig_new_instance_method(_fifechan.CurveGraph_getPointVector)
    resetPointVector = _swig_new_instance_method(_fifechan.CurveGraph_resetPointVector)
    setThickness = _swig_new_instance_method(_fifechan.CurveGraph_setThickness)
    getThickness = _swig_new_instance_method(_fifechan.CurveGraph_getThickness)
    setAutomaticControllPoints = _swig_new_instance_method(_fifechan.CurveGraph_setAutomaticControllPoints)
    isAutomaticControllPoints = _swig_new_instance_method(_fifechan.CurveGraph_isAutomaticControllPoints)
    setOpaque = _swig_new_instance_method(_fifechan.CurveGraph_setOpaque)
    isOpaque = _swig_new_instance_method(_fifechan.CurveGraph_isOpaque)

# Register CurveGraph in _fifechan:
_fifechan.CurveGraph_swigregister(CurveGraph)
class LineGraph(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.LineGraph_swiginit(self, _fifechan.new_LineGraph(*args))
    __swig_destroy__ = _fifechan.delete_LineGraph
    setPointVector = _swig_new_instance_method(_fifechan.LineGraph_setPointVector)
    getPointVector = _swig_new_instance_method(_fifechan.LineGraph_getPointVector)
    resetPointVector = _swig_new_instance_method(_fifechan.LineGraph_resetPointVector)
    setThickness = _swig_new_instance_method(_fifechan.LineGraph_setThickness)
    getThickness = _swig_new_instance_method(_fifechan.LineGraph_getThickness)
    setOpaque = _swig_new_instance_method(_fifechan.LineGraph_setOpaque)
    isOpaque = _swig_new_instance_method(_fifechan.LineGraph_isOpaque)

# Register LineGraph in _fifechan:
_fifechan.LineGraph_swigregister(LineGraph)
class PieGraph(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.PieGraph_swiginit(self, _fifechan.new_PieGraph(*args))
    __swig_destroy__ = _fifechan.delete_PieGraph
    setCenterX = _swig_new_instance_method(_fifechan.PieGraph_setCenterX)
    setCenterY = _swig_new_instance_method(_fifechan.PieGraph_setCenterY)
    getCenterX = _swig_new_instance_method(_fifechan.PieGraph_getCenterX)
    getCenterY = _swig_new_instance_method(_fifechan.PieGraph_getCenterY)
    setCenter = _swig_new_instance_method(_fifechan.PieGraph_setCenter)
    getCenter = _swig_new_instance_method(_fifechan.PieGraph_getCenter)
    setRadius = _swig_new_instance_method(_fifechan.PieGraph_setRadius)
    getRadius = _swig_new_instance_method(_fifechan.PieGraph_getRadius)
    addSegment = _swig_new_instance_method(_fifechan.PieGraph_addSegment)
    clearSegments = _swig_new_instance_method(_fifechan.PieGraph_clearSegments)
    setOpaque = _swig_new_instance_method(_fifechan.PieGraph_setOpaque)
    isOpaque = _swig_new_instance_method(_fifechan.PieGraph_isOpaque)

# Register PieGraph in _fifechan:
_fifechan.PieGraph_swigregister(PieGraph)
class PointGraph(Widget):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        _fifechan.PointGraph_swiginit(self, _fifechan.new_PointGraph(*args))
    __swig_destroy__ = _fifechan.delete_PointGraph
    setPointVector = _swig_new_instance_method(_fifechan.PointGraph_setPointVector)
    getPointVector = _swig_new_instance_method(_fifechan.PointGraph_getPointVector)
    resetPointVector = _swig_new_instance_method(_fifechan.PointGraph_resetPointVector)
    setThickness = _swig_new_instance_method(_fifechan.PointGraph_setThickness)
    getThickness = _swig_new_instance_method(_fifechan.PointGraph_getThickness)
    setOpaque = _swig_new_instance_method(_fifechan.PointGraph_setOpaque)
    isOpaque = _swig_new_instance_method(_fifechan.PointGraph_isOpaque)

# Register PointGraph in _fifechan:
_fifechan.PointGraph_swigregister(PointGraph)
class FcnPointVector(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    iterator = _swig_new_instance_method(_fifechan.FcnPointVector_iterator)
    def __iter__(self):
        return self.iterator()
    __nonzero__ = _swig_new_instance_method(_fifechan.FcnPointVector___nonzero__)
    __bool__ = _swig_new_instance_method(_fifechan.FcnPointVector___bool__)
    __len__ = _swig_new_instance_method(_fifechan.FcnPointVector___len__)
    __getslice__ = _swig_new_instance_method(_fifechan.FcnPointVector___getslice__)
    __setslice__ = _swig_new_instance_method(_fifechan.FcnPointVector___setslice__)
    __delslice__ = _swig_new_instance_method(_fifechan.FcnPointVector___delslice__)
    __delitem__ = _swig_new_instance_method(_fifechan.FcnPointVector___delitem__)
    __getitem__ = _swig_new_instance_method(_fifechan.FcnPointVector___getitem__)
    __setitem__ = _swig_new_instance_method(_fifechan.FcnPointVector___setitem__)
    pop = _swig_new_instance_method(_fifechan.FcnPointVector_pop)
    append = _swig_new_instance_method(_fifechan.FcnPointVector_append)
    empty = _swig_new_instance_method(_fifechan.FcnPointVector_empty)
    size = _swig_new_instance_method(_fifechan.FcnPointVector_size)
    swap = _swig_new_instance_method(_fifechan.FcnPointVector_swap)
    begin = _swig_new_instance_method(_fifechan.FcnPointVector_begin)
    end = _swig_new_instance_method(_fifechan.FcnPointVector_end)
    rbegin = _swig_new_instance_method(_fifechan.FcnPointVector_rbegin)
    rend = _swig_new_instance_method(_fifechan.FcnPointVector_rend)
    clear = _swig_new_instance_method(_fifechan.FcnPointVector_clear)
    get_allocator = _swig_new_instance_method(_fifechan.FcnPointVector_get_allocator)
    pop_back = _swig_new_instance_method(_fifechan.FcnPointVector_pop_back)
    erase = _swig_new_instance_method(_fifechan.FcnPointVector_erase)

    def __init__(self, *args):
        _fifechan.FcnPointVector_swiginit(self, _fifechan.new_FcnPointVector(*args))
    push_back = _swig_new_instance_method(_fifechan.FcnPointVector_push_back)
    front = _swig_new_instance_method(_fifechan.FcnPointVector_front)
    back = _swig_new_instance_method(_fifechan.FcnPointVector_back)
    assign = _swig_new_instance_method(_fifechan.FcnPointVector_assign)
    resize = _swig_new_instance_method(_fifechan.FcnPointVector_resize)
    insert = _swig_new_instance_method(_fifechan.FcnPointVector_insert)
    reserve = _swig_new_instance_method(_fifechan.FcnPointVector_reserve)
    capacity = _swig_new_instance_method(_fifechan.FcnPointVector_capacity)
    __swig_destroy__ = _fifechan.delete_FcnPointVector

# Register FcnPointVector in _fifechan:
_fifechan.FcnPointVector_swigregister(FcnPointVector)




# -*- coding: utf-8 -*-

# ####################################################################
#  Copyright (C) 2005-2019 by the FIFE team
#  http://www.fifengine.net
#  This file is part of FIFE.
#
#  FIFE is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the
#  Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
# ####################################################################

from __future__ import absolute_import
from fife import fifechan

from fife.extensions.pychan.attrs import IntAttr, FloatAttr

from .widget import Widget


class Slider(Widget):
	""" A slider widget

	Use a callback to read out the slider value every time the marker
	is moved.

	New Attributes
	==============

	  - orientation: 1 = horizontal, 0=vertical
	  - scale_start: float: default 0.0
	  - scale_end: float: default 1.0
	  - step_length: float: default scale_end/10
	  - marker_length: int: default 10
	FIXME:
		- update docstrings
	"""

	HORIZONTAL = fifechan.Slider.Horizontal
	VERTICAL = fifechan.Slider.Vertical

	ATTRIBUTES = Widget.ATTRIBUTES + [ IntAttr('orientation'),
									   FloatAttr('scale_start'),
									   FloatAttr('scale_end'),
									   FloatAttr('step_length'),
									   IntAttr('marker_length')
									 ]
	DEFAULT_HEXPAND = True
	DEFAULT_VEXPAND = False
	
	DEFAULT_SIZE = 10,10
	DEFAULT_MIN_SIZE = 10,10

	DEFAULT_SCALE_START = 0.0
	DEFAULT_SCALE_END = 1.0
	DEFAULT_STEP_LENGTH = 0.1
	DEFAULT_MARKER_LENGTH = 10
	DEFAULT_ORIENTATION = HORIZONTAL

	def __init__(self,
				 parent = None,
				 name = None,
				 size = None,
				 min_size = None,
				 max_size = None,
				 fixed_size = None,
				 margins = None,
				 padding = None,
				 helptext = None,
				 position = None,
				 style = None,
				 hexpand = None,
				 vexpand = None,
				 font = None,
				 base_color = None,
				 background_color = None,
				 foreground_color = None,
				 selection_color = None,
				 border_color = None,
				 outline_color = None,
				 border_size = None,
				 outline_size = None,
				 position_technique = None,
				 is_focusable = None,
				 comment = None,
				 scale_start = None,
				 scale_end = None,
				 step_length = None,
				 marker_length = None,
				 orientation = None):

		self.real_widget = fifechan.Slider(scale_start or self.DEFAULT_SCALE_START, scale_end or self.DEFAULT_SCALE_END)
		self.orientation = self.DEFAULT_ORIENTATION
		self.step_length = self.DEFAULT_STEP_LENGTH
		self.marker_length = self.DEFAULT_MARKER_LENGTH

		super(Slider, self).__init__(parent=parent,
									 name=name,
									 size=size,
									 min_size=min_size,
									 max_size=max_size,
									 fixed_size=fixed_size,
									 margins=margins,
									 padding=padding,
									 helptext=helptext,
									 position=position,
									 style=style,
									 hexpand=hexpand,
									 vexpand=vexpand,
									 font=font,
									 base_color=base_color,
									 background_color=background_color,
									 foreground_color=foreground_color,
									 selection_color=selection_color,
									 border_color=border_color,
									 outline_color=outline_color,
									 border_size=border_size,
									 outline_size=outline_size,
									 position_technique=position_technique,
									 is_focusable=is_focusable,
									 comment=comment)

		if orientation is not None: self.orientation = orientation
		if scale_start is not None: self.scale_start = scale_start
		if scale_end is not None: self.scale_end = scale_end
		if step_length is not None: self.step_length = step_length
		if marker_length is not None: self.marker_length = marker_length

		self.accepts_data = True
		self._realSetData = self._setValue
		self._realGetData = self._getValue

	def clone(self, prefix):
		sliderClone = Slider(None,
					self._createNameWithPrefix(prefix),
					self.size,
					self.min_size,
					self.max_size,
					self.fixed_size,
					self.margins,
					self.padding,
					self.helptext,
					self.position,
					self.style,
					self.hexpand,
					self.vexpand,
					self.font,
					self.base_color,
					self.background_color,
					self.foreground_color,
					self.selection_color,
					self.border_color,
					self.outline_color,
					self.border_size,
					self.outline_size,
					self.position_technique,
					self.is_focusable,
					self.comment,
					self.scale_start,
					self.scale_end,
					self.step_length,
					self.marker_length,
					self.orientation)
		return sliderClone

	def _setScale(self, start, end):
		"""setScale(self, double scaleStart, double scaleEnd)"""
		if type(start) != float:
			raise RuntimeError("Slider expects float for start scale")
		if type(end) != float:
			raise RuntimeError("Slider expects float for end scale")
		self.real_widget.setScale(start, end)

	def _getScaleStart(self):
		"""getScaleStart(self) -> double"""
		return self.real_widget.getScaleStart()

	def _setScaleStart(self, start):
		"""setScaleStart(self, double scaleStart)"""
		if type(start) != float:
			raise RuntimeError("Slider expects float for start scale")
		self.real_widget.setScaleStart(start)
	scale_start = property(_getScaleStart, _setScaleStart)

	def _getScaleEnd(self):
		"""getScaleEnd(self) -> double"""
		return self.real_widget.getScaleEnd()

	def _setScaleEnd(self, end):
		"""setScaleEnd(self, double scaleEnd)"""
		if type(end) != float:
			raise RuntimeError("Slider expects float for end scale")
		self.real_widget.setScaleEnd(end)
	scale_end = property(_getScaleEnd, _setScaleEnd)

	def _getValue(self):
		"""getValue(self) -> double"""
		return self.real_widget.getValue()

	def _setValue(self, value):
		"""setValue(self, double value)"""
		if type(value) != float:
			raise RuntimeError("Slider only accepts float values")
		self.real_widget.setValue(value)
	value = property(_getValue, _setValue)

	def _setMarkerLength(self, length):
		"""setMarkerLength(self, int length)"""
		if type(length) != int:
			raise RuntimeError("Slider only accepts int for Marker length")
		self.real_widget.setMarkerLength(length)

	def _getMarkerLength(self):
		"""getMarkerLength(self) -> int"""
		return self.real_widget.getMarkerLength()
	marker_length = property(_getMarkerLength, _setMarkerLength)

	def _setOrientation(self, orientation):
		"""setOrientation(self, Orientation orientation)"""
		self.real_widget.setOrientation(orientation)

	def _getOrientation(self):
		"""getOrientation(self) -> int"""
		return self.real_widget.getOrientation()
	orientation = property(_getOrientation, _setOrientation)

	def _setStepLength(self, length):
		"""setStepLength(self, double length)"""
		if type(length) != float:
			raise RuntimeError("Slider only accepts floats for step length")
		self.real_widget.setStepLength(length)

	def _getStepLength(self):
		"""getStepLength(self) -> double"""
		return self.real_widget.getStepLength()
	step_length = property(_getStepLength, _setStepLength)




# -*- coding: utf-8 -*-

# ####################################################################
#  Copyright (C) 2005-2019 by the FIFE team
#  http://www.fifengine.net
#  This file is part of FIFE.
#
#  FIFE is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the
#  Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
# ####################################################################

from __future__ import absolute_import
from builtins import object
from builtins import str as new_str
from fife import fife, fifechan
from .exceptions import RuntimeError

def get_manager():
	from fife.extensions import pychan
	return pychan.manager

"""
Property bindings
=================

This module contains a set of property bindings for
the widgets, factored out to de-clutter the Widget.

"""
class WrappedProperty(object):
	def __init__(self, name):
		self.name = name
	def _getSetter(self,obj):
		setter_name = 'set' + self.name
		return getattr(obj.real_widget,setter_name)
	def _getGetter(self,obj):
		getter_name = 'get' + self.name
		return getattr(obj.real_widget,getter_name)


class ColorProperty(WrappedProperty):
	"""
	A color property. Fakes a color attribute of a fifechan widget.
	This accepts either tuples of the colors (r,g,b)
	or L{fife.Color} objects.

	Color objects have value semantics in this case.
	"""
	def __init__(self, name):
		super(ColorProperty, self).__init__(name)
	def __set__(self, obj, color):
		if isinstance(color, tuple):
			color = fifechan.Color(*color)
		else:
			# Force a copy to get value semantics
			color = fifechan.Color(color.r,color.g,color.b,color.a)
		self._getSetter(obj)(color)
	def __get__(self, obj, objtype = None):
		color = self._getGetter(obj)()
		return fifechan.Color(color.r,color.g,color.b,color.a)

class ImageProperty(WrappedProperty):
	"""
	This unifies the way Images and Image attributes are handled
	inside PyChan.
	"""
	def __init__(self, name):
		super(ImageProperty, self).__init__(name)
		self.prop_name = "_prop_" + self.name.lower()
	def __set__(self, obj, image):
		image_info = getattr(obj, self.prop_name, {})
		if not image:
			image_info["source"] = ""
			image_info["image"] = fife.GuiImage()
			image_info["image"]._source = ""
			self._getSetter(obj)(None)

		elif isinstance(image, (str, new_str)):
			image_info["source"] = image
			# to catch or not to catch ...
			# we just let the NotFound exception trickle here.
			# depedning on complains we can catch and print a warning.
			image_info["image"] = get_manager().loadImage(image)
			image_info["image"].source = image
			self._getSetter(obj)(image_info["image"])

		elif isinstance(image,fife.GuiImage):
			# FIXME - this trickery with the hidden annotation
			# with an _source attribute isn't really clean.
			# Is it even necessary
			image_info["source"] = getattr(image,"source","")
			image_info["image"] = image
			if image_info["source"]:
				image_info["image"] = get_manager().loadImage(image_info["source"])
			self._getSetter(obj)(image_info["image"])
		else:
			attribute_name = "%s.%s" % (obj.__class__.__name__,self.name)
			error_message = "%s only accepts GuiImage and python strings, not '%s'"
			raise RuntimeError(error_message % (attribute_name, repr(image)))
		
		setattr(obj, self.prop_name, image_info)

	def __get__(self, obj, objtype = None):
		d = getattr(obj, self.prop_name, {})
		image = d.get("image",None)
		if not image:
			image = fife.GuiImage()
			image.source = ""
		return image




# -*- coding: utf-8 -*-

# ####################################################################
#  Copyright (C) 2005-2013 by the FIFE team
#  http://www.fifengine.net
#  This file is part of FIFE.
#
#  FIFE is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the
#  Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
# ####################################################################

from __future__ import absolute_import
from fife import fife
from fife import fifechan

from fife.extensions.pychan.internal import get_manager
from fife.extensions.pychan.attrs import Attr, BoolAttr
from fife.extensions.serializers.xmlanimation import loadXMLAnimation

from .widget import Widget


class AnimationIcon(Widget):
	"""
	An animation icon.

	New Attributes
	==============

	  - animation: String : The source location of the Animation xml.
	  - scale: Boolean: True if the image should be scaled to widget size, false otherwise.
	  - tile: Boolean: True if the image should be tiled to widget size, false otherwise.
	  - opaque: Boolean: True if the background of the icon should be drawn, false otherwise.
	"""
	ATTRIBUTES = Widget.ATTRIBUTES + [ Attr('animation'),
									   BoolAttr('scale'),
									   BoolAttr('tile'),
									   BoolAttr('opaque'),
									   BoolAttr('repeating'),
									   BoolAttr('play')
									 ]
	DEFAULT_MARGINS = 0, 0
	DEFAULT_PADDING = 0

	DEFAULT_SCALE = False
	DEFAULT_TILE = False
	DEFAULT_OPAQUE = False
	DEFAULT_REPEATING = True
	DEFAULT_PLAY = True
	
	def __init__(self, 
				 parent = None,
				 name = None,
				 size = None,
				 min_size = None,
				 max_size = None,
				 fixed_size = None,
				 margins = None,
				 padding = None,
				 helptext = None,
				 position = None,
				 style = None,
				 hexpand = None,
				 vexpand = None,
				 font = None,
				 base_color = None,
				 background_color = None,
				 foreground_color = None,
				 selection_color = None,
				 border_color = None,
				 outline_color = None,
				 border_size = None,
				 outline_size = None,
				 position_technique = None,
				 is_focusable = None,
				 comment = None,
				 animation = None,
				 scale = None,
				 tile = None,
				 opaque = None,
				 repeating = None,
				 play = None):
				 
		self.real_widget = fifechan.AnimationIcon()
		self._anim = None
		
		super(AnimationIcon,self).__init__(parent=parent,
								  name=name,
								  size=size,
								  min_size=min_size,
								  max_size=max_size,
								  fixed_size=fixed_size,
								  margins=margins,
								  padding=padding,
								  helptext=helptext,
								  position=position,
								  style=style,
								  hexpand=hexpand,
								  vexpand=vexpand,
								  font=font,
								  base_color=base_color,
								  background_color=background_color,
								  foreground_color=foreground_color,
								  selection_color=selection_color,
								  border_color=border_color,
								  outline_color=outline_color,
								  border_size=border_size,
								  outline_size=outline_size,
								  position_technique=position_technique,
								  is_focusable=is_focusable,
								  comment=comment)

		# set provided attributes or defaults
		if scale is not None: self.scale = scale
		else: self.scale = self.DEFAULT_SCALE

		if tile is not None: self.tile = tile
		else: self.tile = self.DEFAULT_TILE

		if opaque is not None: self.opaque = opaque
		else: self.opaque = self.DEFAULT_OPAQUE

		# for the case that animation can not be found, e.g. invalid path
		# the AnimationIcon is removed from the manager
		try:
			self.animation = animation
		except Exception:
			get_manager().removeWidget(self)
			raise

		if repeating is not None: self.repeating = repeating
		else: self.repeating = self.DEFAULT_REPEATING

		if play is not None: self.play = play
		else: self.play = self.DEFAULT_PLAY
		
		#if the size parameter is specified set it (again) to override
		#the icons size. That works only without layouting.
		if size is not None: self.size = size

	def clone(self, prefix):
		iconClone = AnimationIcon(None,
				self._createNameWithPrefix(prefix),
				self.size,
				self.min_size, 
				self.max_size,
				self.fixed_size,
				self.margins,
				self.padding,
				self.helptext, 
				self.position, 
				self.style, 
				self.hexpand,
				self.vexpand,
				self.font,
				self.base_color,
				self.background_color,
				self.foreground_color,
				self.selection_color,
				self.border_color,
				self.outline_color,
				self.border_size,
				self.outline_size,
				self.position_technique,
				self.is_focusable,
				self.comment,
				self.animation,
				self.scale,
				self.tile,
				self.opaque,
				self.repeating,
				self.play)
						
		return iconClone

	def _setAnimation(self, anim):
		if anim is not None:
			if isinstance(anim, fife.Animation):
				self._anim = anim
			else:
				if anim != "":
					# use xml loader
					self._anim = loadXMLAnimation(get_manager().hook.engine, anim)
			self.real_widget.setAnimation(self._anim)
	def _getAnimation(self):
		return self._anim
	animation = property(_getAnimation, _setAnimation)

	def _setScaling(self, val):	self.real_widget.setScaling(val)
	def _getScaling(self): return self.real_widget.isScaling()
	scale = property(_getScaling, _setScaling)

	def _setTiling(self, val):	self.real_widget.setTiling(val)
	def _getTiling(self): return self.real_widget.isTiling()
	tile = property(_getTiling, _setTiling)

	def _setOpaque(self, val):	self.real_widget.setOpaque(val)
	def _getOpaque(self): return self.real_widget.isOpaque()
	opaque = property(_getOpaque, _setOpaque)

	def _setRepeating(self, repeat): self.real_widget.setRepeating(repeat)
	def _getRepeating(self): return self.real_widget.isRepeating()
	repeating = property(_getRepeating, _setRepeating)
	
	def startPlaying(self):
		self.real_widget.play()

	def isPlaying(self):
		return self.real_widget.isPlaying()

	def pausePlaying(self):
		self.real_widget.pause()

	def stopPlaying(self):
		self.real_widget.stop()

	def _setPlaying(self, play):
		if play is True: self.startPlaying()
		else: self.stopPlaying()
	def _getPlaying(self): return self.isPlaying()
	play = property(_getPlaying, _setPlaying)




# -*- coding: utf-8 -*-

# ####################################################################
#  Copyright (C) 2005-2019 by the FIFE team
#  http://www.fifengine.net
#  This file is part of FIFE.
#
#  FIFE is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the
#  Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
# ####################################################################

from __future__ import print_function
from builtins import object
from fife import fife

class LogManager(object):
	"""
	Log manager provides convenient apis to access engine logging functionality.
	You can set log targets individually (prompt, file). You can also adjust
	things like visible modules through log manager.
	"""

	def __init__(self, engine, promptlog=True, filelog=False):
		"""
		Constructs new log manager
		@param engine: Engine to hook into
		@param promptlog: If true, logs to prompt
		@param filelog: If true, logs to file (fife.log)
		"""
		self.engine = engine
		self.lm = engine.getLogManager()
		self.lm.setLogToPrompt(promptlog)
		self.lm.setLogToFile(filelog)
		self.mod2name = {}
		for k, v in list(fife.__dict__.items()):
			if k.startswith('LM_') and k not in ('LM_CORE', 'LM_MODULE_MAX'):
				self.mod2name[v] = self.lm.getModuleName(v)
		self.name2mod = dict([(v.lower(), k) for k, v in list(self.mod2name.items())])

	def addVisibleModules(self, *names):
		"""
		Adds modules that are visible in logs. By default, all modules
		are disabled. Does not remove previously visible modules
		@param names: module names to set visible
		@see: modules.h file for available modules in the engine
		"""
		names = [n.lower() for n in names]
		if 'all' in names:
			for k in list(self.mod2name.keys()):
				self.lm.addVisibleModule(k)
		else:
			for m in names:
				try:
					self.lm.addVisibleModule(self.name2mod[m])
				except KeyError:
					print('Tried to enable non-existing log module "%s"' % m)

	def removeVisibleModules(self, *names):
		"""
		Removes modules that are visible in logs. By default, all modules
		are disabled.
		@param names: module names to set invisible
		@see: addVisibleModules
		"""
		names = [n.lower() for n in names]
		if 'all' in names:
			for k in list(self.mod2name.keys()):
				self.lm.removeVisibleModule(k)
		else:
			for m in names:
				self.lm.removeVisibleModule(self.name2mod[m])

	def getVisibleModules(self):
		"""
		Gets currently visible modules
		@see: addVisibleModules
		"""
		mods = []
		for k in list(self.mod2name.keys()):
			if self.lm.isVisible(k):
				mods.append(self.mod2name[k])

	def setVisibleModules(self, *names):
		"""
		Sets visible modules. Clears previously set modules.
		@param names: module names to set visible
		@see: addVisibleModules
		"""
		self.lm.clearVisibleModules()
		self.addVisibleModules(*names)

	def setLevelFilter(self, fltr):
		"""
		Sets the minimum log level to view.

		@param fltr: The filter level

		Valid values:
			- L{fife.LogManager.LEVEL_DEBUG}
			- L{fife.LogManager.LEVEL_LOG}
			- L{fife.LogManager.LEVEL_WARN}
   			- L{fife.LogManager.LEVEL_ERROR}
		"""
		self.lm.setLevelFilter(fltr)

	def setLogToPrompt(self, promptlog):
		return self.lm.setLogToPrompt(promptlog)

	def getLogToPrompt(self):
		return self.lm.isLogToPrompt()

	logToPrompt = property(getLogToPrompt, setLogToPrompt)

	def setLogToFile(self, filelog):
		return self.lm.setLogToFile(filelog)

	def getLogToFile(self):
		return self.lm.isLogToFile()

	logToFile = property(getLogToFile, setLogToFile)

	def log_debug(self, message):
		self.lm.log(0, self.name2mod["script"], message)

	def log_log(self, message):
		self.lm.log(1, self.name2mod["script"], message)

	def log_warn(self, message):
		self.lm.log(2, self.name2mod["script"], message)

	def log_error(self, message):
		self.lm.log(3, self.name2mod["script"], message)



# -*- coding: utf-8 -*-

# ####################################################################
#  Copyright (C) 2005-2019 by the FIFE team
#  http://www.fifengine.net
#  This file is part of FIFE.
#
#  FIFE is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the
#  Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
# ####################################################################

from __future__ import absolute_import
from builtins import str
from builtins import map
from builtins import range
from fife import fifechan

from .common import gui2str, text2gui
from .widget import Widget

class GenericListmodel(fifechan.ListModel,list):
	"""
	A wrapper for the exported list model to behave more like a Python list.
	Don't use directly.
	"""
	def __init__(self,*args):
		super(GenericListmodel,self).__init__()
		list(map(self.append,args))
	def clear(self):
		while len(self):
			self.pop()
	def getNumberOfElements(self):
		return len(self)

	def getElementAt(self, i):
		i = max(0,min(i,len(self) - 1))
		return text2gui(str(self[i]))

class ListBox(Widget):
	"""
	A basic list box widget for displaying lists of strings. It makes most sense to wrap
	this into a L{ScrollArea}.

	New Attributes
	==============

	  - items: A List of strings. This can be treated like an ordinary python list.
	    but only strings are allowed.
	  - selected: The index of the selected item in the list. Starting from C{0} to C{len(items)-1}.
	    A negative value indicates, that no item is selected.
	  - selected_item: The selected string itself, or C{None} - if no string is selected.

	Data
	====
	The selected attribute can be read and set via L{distributeData} and L{collectData}.
	The list items can be set via L{distributeInitialData}.
	"""
	DEFAULT_HEXPAND = True
	DEFAULT_VEXPAND = True
	DEFAULT_ITEMS = []
	
	def __init__(self, 
				 parent = None,
				 name = None,
				 size = None,
				 min_size = None,
				 max_size = None,
				 fixed_size = None,
				 margins = None,
				 padding = None,
				 helptext = None,
				 position = None,
				 style = None,
				 hexpand = None,
				 vexpand = None,
				 font = None,
				 base_color = None,
				 background_color = None,
				 foreground_color = None,
				 selection_color = None,
				 border_color = None,
				 outline_color = None,
				 border_size = None,
				 outline_size = None,
				 position_technique = None,
				 is_focusable = None,
				 comment = None,
				 items = None,
				 selected = None):
				 
		if items is None:
			self._items = GenericListmodel(*self.DEFAULT_ITEMS)
		else:
			if type(items) is list:
				self._items = GenericListmodel(*items)
			else:
				raise RuntimeError("items parameter must be a list!")
			
		self.real_widget = fifechan.ListBox(self._items)
		
		if selected is not None and items is not None:
			if len(items) > 0 and selected < len(items) and selected > 0:
				self.selected = selected
			else:
				raise RuntimeError("Invalid selected item index specified for listbox!")
		elif selected is None and items is not None:
			if len(items) > 0:
				self.selected = 0
				
		super(ListBox,self).__init__(parent=parent, 
									 name=name, 
									 size=size, 
									 min_size=min_size, 
									 max_size=max_size,
									 fixed_size=fixed_size,
									 margins=margins,
									 padding=padding,
									 helptext=helptext, 
									 position=position,
									 style=style, 
									 hexpand=hexpand, 
									 vexpand=vexpand,
									 font=font,
									 base_color=base_color,
									 background_color=background_color,
									 foreground_color=foreground_color,
									 selection_color=selection_color,
									 border_color=border_color,
									 outline_color=outline_color,
									 border_size=border_size,
									 outline_size=outline_size,
									 position_technique=position_technique,
									 is_focusable=is_focusable,
									 comment=comment)

		# Prepare Data collection framework
		self.accepts_initial_data = True
		self._realSetInitialData = self._setItems

		self.accepts_data = True
		self._realSetData = self._setSelected
		self._realGetData = self._getSelected

	def clone(self, prefix):
		
		itemsList = []
		
		for i in range(self.items.getNumberOfElements()):
			itemsList.append(self.items.getElementAt(i))
		
		listboxClone = ListBox(None, 
					self._createNameWithPrefix(prefix),
					self.size,
					self.min_size,
					self.max_size,
					self.fixed_size,
					self.margins,
					self.padding,
					self.helptext,
					self.position,
					self.styleNone,
					self.hexpand,
					self.vexpand,
					self.font,
					self.base_color,
					self.background_color,
					self.foreground_color,
					self.selection_color,
					self.border_color,
					self.outline_color,
					self.border_size,
					self.outline_size,
					self.position_technique,
					self.is_focusable,
					self.comment,
					itemsList,
					self.selected)			
					
		return listboxClone
				
	def _getItems(self): return self._items #self.real_widget.getListModel() works too
	def _setItems(self,items):
		# Also self assignment can kill you but
		# without the GenericListmodel is freed instantly ;-)
		if id(items) != id(self._items):
			self._items.clear()
			self._items.extend(items)
			self.real_widget.setListModel(self._items)
	items = property(_getItems,_setItems)

	def _getSelected(self): return self.real_widget.getSelected()
	def _setSelected(self,index): self.real_widget.setSelected(index)
	selected = property(_getSelected,_setSelected)
	def _getSelectedItem(self):
		if 0 <= self.selected < len(self._items):
			return self._items[self.selected]
		return None
	selected_item = property(_getSelectedItem)
